<?php
namespace Teknisa\CMP\Modules\Simplificada\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Setforms2\Shared\ORM\DateTime;
use Exception;

class CompraSimplificadaController extends \Teknisa\GER\Controllers\TeknisaCrud
{
    protected $crudService;
    protected $parameterBag;
    protected $simplificadaService;
    protected $dataSourceManager;
    protected $entityManager;
    protected $necessidadeController;
    
    protected $parasupri;
    protected $parcmpsup;
    protected $filial;
    protected $idcalprevcon;
    protected $startnececomp;
    
    public function __construct($crudService, $parameterBag, $simplificadaService, $dataSourceManager, EntityManager $entityManager, $necessidadeController)
    {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->simplificadaService = $simplificadaService;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager = $entityManager;
        $this->necessidadeController = $necessidadeController;
    }
    
    public function getCompraSimplificada(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = array(
                array(
                    "columnName" => "NRORG",
                    "operator"   => "=",
                    "value"      => $this->simplificadaService->getNrorg()
                )
            );
            
            $filtro = $this->crudService->getFilteredDataSet('compra_simplificada', $conditions, $request->getFilterCriteria()->getPage(), $request->getFilterCriteria()->getPageSize())->getRows();

            $response->addDataSet(new DataSet('filtro', $filtro));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getFilialCompraSimplificada(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
        
            array_push($conditions, [
                'columnName' => 'NRORG',
                'value' =>  $this->simplificadaService->getNrorg(),
                'operator' => '=',
            ]);

            $filtro = $this->crudService->getFilteredDataSet('filial_compra_simplificada', $conditions, $request->getFilterCriteria()->getPage(), $request->getFilterCriteria()->getPageSize())->getRows();
            
            $response->addDataSet(new DataSet('filtro', $filtro));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getProdutoCompraSimplificada(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
        
            array_push($conditions, [
                'columnName' => 'NRORG', 
                'value' =>  $this->simplificadaService->getNrorg(),
                'operator' => '=',
            ]);

            $filtro = $this->crudService->getFilteredDataSet('produto_compra_simplificada', $conditions, $request->getFilterCriteria()->getPage(), $request->getFilterCriteria()->getPageSize())->getRows();
            
            $response->addDataSet(new DataSet('filtro', $filtro));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getFilialOper(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
        
            array_push($conditions, [
                'columnName' => 'NRORG',
                'value' =>  $this->simplificadaService->getNrorg(),
                'operator' => '=',
            ],[
                'columnName' => 'CDOPERADOR',
                'value' =>  $this->simplificadaService->getOperadorLogado(),
                'operator' => '=',
            ]);

            $filtro = $this->crudService->getFilteredDataSet('filial_filioper', $conditions, $request->getFilterCriteria()->getPage(), $request->getFilterCriteria()->getPageSize())->getRows();
            
            $response->addDataSet(new DataSet('filtro', $filtro));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveCompraSimplificada(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        
        try {
            $this->simplificadaService->iniciaTransacao();
            
            $nrfiltro = $this->simplificadaService->saveCompraSimplificadaFilter();
            
            foreach($conditions as $condition) {
                if($condition['columnName'] == 'P_CDFILSOLI') {
                    foreach($condition['value'] as $filial) {
                        $this->simplificadaService->saveUnidadeSimplificadaFilter($nrfiltro, $filial);
                    }
                }
                if($condition['columnName'] == 'P_CDNVPRODUTO') {
                    $nvproduto = $condition['value']   ;
                }
                if($condition['columnName'] == 'P_CDPRODUTO') {
                    foreach($condition['value'] as $produto) {
                        $this->simplificadaService->saveProdutoSimplificadaFilter($nrfiltro, $produto, $nvproduto);
                    }
                }
                if($condition['columnName'] == 'P_DTENTRINI') {
                    $dtIni = $condition['value'];
                }
                if($condition['columnName'] == 'P_DTENTRFIM') {
                    $dtFim = $condition['value'];
                }
            }
            
            $periodicidade = $this->serviceCompraSimplificada->getDataEntrega($nrfiltro);
            if(is_null($periodicidade)) {
                $result = $periodicidade;
            } else {
                
            }
            
            $this->simplificadaService->flush();
            $this->simplificadaService->commit();
            
            $response->addDataSet(new DataSet('result', $result));
        } catch(\Exception $e) {
            $this->simplificadaService->rollback();
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function deleteCompraSimplificada(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $this->simplificadaService->iniciaTransacao();
            
            $row = $request->getRow();
        
            if (!empty($row)) {
                foreach ($row as $value) {
                    $this->simplificadaService->deleteCompraSimplificadaFilter($value['NRFILTROCOMPSFDA']);
                }
            }
            
            $this->simplificadaService->flush();
            $this->simplificadaService->commit();
        } catch(\Exception $e) {
            $this->simplificadaService->rollback();
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    
    /*
	*
	* To do 
	* Implementar rotina de chamada da necessidade de compras
	*
	*
	*
	*/
    
    private function getNeceCompra($nrcmpsplf)
    {
        $datafimsegu = $this->necessidadeController->getDataLimiteSeguranca('', '', '', '', "31/01/2020", $nrcmpsplf);
        
        $filial     = $this->necessidadeController->getFilial($this->simplificadaService->getFilialLogada());
        $parasupri  = $this->necessidadeController->getParaSupri($filial['CDFILCENTCMP']);
        $parcmpsup  = $this->necessidadeController->getParCmpSup($this->simplificadaService->getFilialLogada());
        
        $idcalprevcon = $parcmpsup['IDCALPREVCON'];
        if($idcalprevcon == 'N') {
            self.CalcNecesComp(row);    
        } 
        return $idcalprevcon;
        
                    /*row.DATFIMSEGU = calback.dataset.resposta.DATSEGU;
                    row.MSGBACK = "Parâmetros default de consumo devem ser informados.";
                    let msg = "Deseja calcular a previsão de consumo no período entre as datas de processamento de cada unidade e o dia " + row.DATFIMSEGU + "?";
                
                    if(idcalprevcon !== 'N'){
                        if(idcalprevcon == 'S'){
                            ScreenService.confirmMessage(msg, 'question', function(success) {
                                General.callRestEngine('cmp_necessidadeCompras#/GetPrxConsPlan','Row',row).then(function(result) {
                                    if(!result.dataset.resposta.MSG){
                                        self.CalcNecesComp(row);
                                    }else{
                                        ScreenService.showMessage(result.dataset.resposta.MSG);
                                    }
                                    
                                }).catch(function(error) {
                                    ScreenService.showMessage(error.data ? error.data.error : 'Tempo limite de conexão excedido.', 'error', 'error');
                                }); 
                            }, function(error) {
                                self.CalcNecesComp(row);
                            });
                        } else{
                            General.callRestEngine('cmp_necessidadeCompras#/GetPrxConsPlan','Row',row).then(function(result) {
                                if(!result.dataset.resposta.MSG){
                                    self.CalcNecesComp(row);
                                }else{
                                    ScreenService.showMessage(result.dataset.resposta.MSG);
                                }
                            }).catch(function(error) {
                                ScreenService.showMessage(error.data ? error.data.error : 'Tempo limite de conexão excedido.', 'error', 'error');
                            }); 
                        }       
                    } else{
                        self.CalcNecesComp(row);
                    }*/
        
        
        $date = new DateTime();
        $date = $date->format('d/m/Y H:i:s');
        
        $datini = new DateTime('01-01-2019');
        $datini = $datini->format('d/m/Y');
        $datfin = new DateTime('31-12-2019');
        $datfin = $datfin->format('d/m/Y');
        //$row['NECECOMP'] = $this->simplificadaService->getNewCode('NECECOMP');
        //$this->simplificadaService->NECESSIDADE_DE_COMPRAS($datini, $datfin, $row['NECECOMP'], $date, $nrcmpsplf);
        //return $this->getConfereNeceComp($row['NECECOMP'], $datini);
    }
}