<?php

namespace Teknisa\CMP\Modules\Solicitacao\Modules\SolicitacaoCompras\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class CMP03100LancamentoDeSolicitacaoDeComprasService {
    
    protected $connection;
    
    public function __construct($entityManager, $environment){
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
    }
    
    public function parseFloat($floatVar){
        if(is_string($floatVar)){
            $floatVar = str_replace(".", "", $floatVar);    
            $floatVar = str_replace(",", ".", $floatVar);
        }
        return $floatVar;
    }
    
    public function retornaCentralCompras($cdfilial){
        $param = array();
        $param['CDFILIAL'] = $cdfilial;
        $param['NRORG']    = $this->environment->getNrOrg();
        
        $query = "  SELECT
                        FILIAL.NRORG,
                        NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AS CDFILCENTCMP,
                        FILCENT.NMFILIAL AS NMFILIAL_CENTRAL
                    FROM
                        FILIAL FILCENT,
                        FILIAL
                    WHERE
                        FILIAL.CDFILIAL = :CDFILIAL AND
                        FILIAL.NRORG    = :NRORG    AND
                        NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) = FILCENT.CDFILIAL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function validaUnidade($cdfilial){
        $param = array();
        $param['CDFILIAL'] = $cdfilial;
        $param['NRORG']    = $this->environment->getNrOrg();
        
        $query = "  SELECT
                        FILIAL.CDFILIAL,
                        FILIAL.NMFILIAL,
                        FILIAL.IDCNTRRESC,
                        NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) CDFILCENTCMP,
                        FILIAL.CDREGIAO,
                        FILIAL.NRORG
                    FROM
                        FILIAL
                    WHERE
                        ((INSTR(:CDFILIAL, FILIAL.CDFILIAL) > 0) OR (:CDFILIAL = 'T')) AND
                        (FILIAL.IDCNTRRESC <> 'S')                       AND
                        (FILIAL.CDREGIAO IS NOT NULL)                    AND
                        (FILIAL.NRORG = :NRORG)
                    ORDER BY
                        FILIAL.CDFILIAL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function retornaParasupri($data){
        $param = array();
        $param['P_CDFILIAL'] = $data;
        $param['NRORG'] = $this->environment->getNrOrg();
        $query = "  SELECT
                       CDFILIAL    , IDJUSSOLEXT , IDBLOQALTNEC, IDAUDALTSOL,
                       IDULTIDTULTI, IDULTCOMPSOL, IDULTDESTSOL, CDGRUPOPER ,
                       IDULTIDTVALQ, IDVALDTAF   , IDBLOQUEAAF , IDMARSEMQUA,
                       IDTIPOAPUR,   IDPROSEMMAR,  IDPROSEMFOR , IDUTQTDEVOLU,
                       IDBAIXSOLDFS, IDCONSPRSOL, IDEXJUSTESCM , IDBLOQALTSOLAUT, NRORG
                    FROM
                       PARASUPRI
                    WHERE
                       (INSTR(:P_CDFILIAL, CDFILIAL) > 0) AND
                       (NRORG = :NRORG)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function filiaisSelecionadas ($data){
        $param = array();
        $param['P_CDFILIAL'] = $data['CDFILIAL'];
        $param['P_TCDFILIAL'] = $data['TCDFILIAL'];
        $param['NRORG'] = $this->environment->getNrOrg();
        $query = "SELECT
                     CDDESTSOLI,
                     MIN (NMDESTSOLI) NMDESTSOLI,
                     NRORG
                  FROM
                     DESTSOL
                  WHERE
                    ((INSTR(:P_CDFILIAL, CDFILIAL) > 0) OR ('T' = :P_TCDFILIAL)) AND
                    (NRORG = :NRORG)
                  GROUP BY 
                    CDDESTSOLI, NRORG
                  ORDER BY 
                    CDDESTSOLI";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function validaAprovacao (){
        $query = "SELECT COUNT(*) CONTADOR FROM APROVSOLPRODGER";
        try {
            $std = $this->connection->prepare($query);
            $std->execute();
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function validaGerente ($operador, $nrorg){
        $param = array();
        $param['P_CDOPERADOR'] = $operador;
        $param['P_TCDOPERADOR'] = "";
        $param['NRORG'] = $this->environment->getNrOrg();
        $query = "SELECT 
                    OPERADOR.CDOPERADOR, OPERADOR.NMOPERADOR, OPERADOR.NRORG 
                 FROM 
                    GERENTE, OPERADOR
                 WHERE 
                    (GERENTE.CDOPERADOR = OPERADOR.CDOPERADOR) AND 
                    ((OPERADOR.CDOPERADOR = :P_CDOPERADOR) OR  (:P_TCDOPERADOR = 'T')) AND
                    GERENTE.NRORG = OPERADOR.NRORG AND
                    OPERADOR.NRORG = :NRORG
                 ORDER BY 
                    OPERADOR.CDOPERADOR, OPERADOR.NRORG, GERENTE.NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function selectFiltro ($row){
        $param = array();
        $param['P_CDFILIAL'] = $row['CDFILIAL'];
        $param['P_CDGRUPFILI'] = $row['CDGRUPFILI'];
        $param['P_DTSOLINI'] = $row['DTSOLINI'];
        $param['P_DTSOLFIM'] = $row['DTSOLFIM'];
        $param['P_TSOLICITA'] = $row['TSOLICITA'];
        $param['P_COMPRADOR'] = $row['COMPRADOR'];
        $param['P_TCOMPRADOR'] = $row['TCOMPRADOR'];
        $param['P_DESTINO'] = $row['CDDESTSOLI'];
        $param['P_TDESTINO'] = $row['TCDDESTSOLI'];
        $param['P_TCDGERENTE'] = $row['TCDGERENTE'];
        $param['P_CDGERENTE'] = $row['CDGERENTE'];
        $param['P_STATUS'] = $row['APROVACAO'];
        $param['P_CDOPERADOR'] = $row['CDOPERADOR'];
        $param['NRORG'] = $this->environment->getNrOrg();
        
        $query = "SELECT DISTINCT
                     FILIAL.CDFILIAL,
                     FILIAL.NMFILIAL,
                     NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AS CDFILCENTCMP,
                     FILCENT.NMFILIAL AS NMCENTCMP,
                     PARASUPRI.IDJUSSOLEXT,
                     PARASUPRI.IDBLOQALTSOLAUT,
                     PARASUPRI.IDAUDALTSOL,
                     PARASUPRI.IDULTIDTULTI,
                     PARASUPRI.IDULTCOMPSOL,
                     PARASUPRI.IDULTDESTSOL,
                     PARASUPRI.IDUTQTDEVOLU,
                     PARASUPRI.IDBAIXSOLDFS,
                     PARCMPSUP.IDUTPRODPREF,
                     PARCMPSUP.IDUTAPROVSOL,
                     PARCMPSUP.IDUTLOPERSOL,
                     PARCMPSUP.IDPERALTSAPR,
                     PARCMPSUP.IDIMPINCSOLAPR
                  FROM
                     PARAMGERAL,
                     PARASUPRI,
                     PARCMPSUP,
                     SOLICITA,
                     FILIAL FILCENT,
                     (SELECT
                         FILIAL.CDFILIAL,
                         FILIAL.CDFILCENTCMP,
                         FILIAL.NMFILIAL,
                         FILIAL.NRORG
                      FROM
                         FILGRUPFILI,
                         FILIAL
                      WHERE
                        (FILIAL.IDCNTRRESC <> 'S')               AND
                        (FILIAL.CDREGIAO IS NOT NULL)            AND
                        (FILGRUPFILI.CDFILIAL = FILIAL.CDFILIAL) AND
                        (FILGRUPFILI.CDGRUPFILI = :P_CDGRUPFILI) AND
                        FILGRUPFILI.NRORG = FILIAL.NRORG  AND
                        FILIAL.NRORG = :NRORG AND
                  
                          (INSTR(:P_CDFILIAL, FILGRUPFILI.CDFILIAL) > 0 OR :P_CDFILIAL = 'T')
                  
                     UNION
                  
                     SELECT
                        CDFILIAL,
                        CDFILCENTCMP,
                        NMFILIAL,
                        NRORG
                     FROM
                        FILIAL
                     WHERE
                        (FILIAL.IDCNTRRESC <> 'S')     AND
                        (FILIAL.CDREGIAO IS NOT NULL)  AND
                        (:P_CDGRUPFILI IS NULL)        AND
                        FILIAL.NRORG = :NRORG AND
                  
                          (INSTR(:P_CDFILIAL, FILIAL.CDFILIAL) > 0 OR :P_CDFILIAL = 'T')) FILIAL
                   WHERE
                       PARAMGERAL.NRORG =  PARASUPRI.NRORG AND
                       PARASUPRI.NRORG  = PARCMPSUP.NRORG AND
                       PARCMPSUP.NRORG  = SOLICITA.NRORG AND
                       SOLICITA.NRORG   = FILCENT.NRORG AND
                       FILCENT.NRORG   = FILIAL.NRORG  AND
                       FILIAL.NRORG    = :NRORG AND
                  
                       (PARASUPRI.CDFILIAL  = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)) AND
                  
                       (PARCMPSUP.CDFILIAL  = FILIAL.CDFILIAL)                           AND
                  
                       (NVL(FILIAL.CDFILCENTCMP,FILIAL.CDFILIAL) = FILCENT.CDFILIAL)     AND
                  
                       (SOLICITA.CDFILSOLI  = FILIAL.CDFILIAL)                           AND
                  
                       ((SOLICITA.DTSOLCMP BETWEEN :P_DTSOLINI AND :P_DTSOLFIM) OR(:P_TSOLICITA = 'T'))  AND
                  
                       (INSTR(:P_COMPRADOR, SOLICITA.CDOPERADOR) <> 0 OR :P_TCOMPRADOR = 'T') AND
                  
                       (INSTR(:P_DESTINO, SOLICITA.CDDESTSOLI) <> 0 OR   :P_TDESTINO = 'T') AND
                     (:P_TCDGERENTE = 'T' OR
                     EXISTS (SELECT 1
                               FROM APROVSOLPRODGER A, DESTSOLGERE D
                              WHERE A.CDFILLANCA = SOLICITA.CDFILLANCA
                                AND A.CDFILSOLI  = SOLICITA.CDFILSOLI
                                AND A.NRSOLICMP  = SOLICITA.NRSOLICMP
                                AND A.CDFILSOLI  = D.CDFILIAL
                                AND A.CDOPERADOR = D.CDOPERADOR
                                AND D.CDDESTSOLI = SOLICITA.CDDESTSOLI
                                AND D.CDOPERADOR = :P_CDGERENTE
                                AND D.NRORG      =  A.NRORG
                                AND A.NRORG      = SOLICITA.NRORG
                  
                                  AND ((INSTR(:P_STATUS, (NVL(A.IDAPROVPRGER, 'X'))) > 0) OR (:P_STATUS =  'T'))
                                )) AND
                     (PARAMGERAL.IDRESTRFILIOPER = 'N' OR
                     EXISTS (SELECT 1
                               FROM FILIOPER
                              WHERE FILIOPER.CDOPERADOR = :P_CDOPERADOR
                                AND FILIOPER.CDFILIAL   =  SOLICITA.CDFILSOLI
                                AND FILIOPER.NRORG = :NRORG))
                  ORDER BY
                    FILIAL.CDFILIAL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function selectFiltroEntrega ($row){
        $param = array();
        $param['P_CDFILIAL']    = $row['CDFILIAL'];
        $param['P_CDGRUPFILI']  = $row['CDGRUPFILI'];
        $param['P_DTSOLINI']    = $row['DTSOLINI'];
        $param['P_DTSOLFIM']    = $row['DTSOLFIM'];
        $param['P_TSOLICITA']   = $row['TSOLICITA'];
        $param['P_COMPRADOR']   = $row['COMPRADOR'];
        $param['P_TCOMPRADOR']  = $row['TCOMPRADOR'];
        $param['P_DESTINO']     = $row['CDDESTSOLI'];
        $param['P_TDESTINO']    = $row['TCDDESTSOLI'];
        $param['P_TCDGERENTE']  = $row['TCDGERENTE'];
        $param['P_CDGERENTE']   = $row['CDGERENTE'];
        $param['P_STATUS']      = $row['APROVACAO'];
        $param['P_CDOPERADOR']  = $row['CDOPERADOR'];
        $param['NRORG']         = $this->environment->getNrOrg();
        
        $param['P_ENTRINI']     = $row['ENTRINI'];
        $param['P_ENTRFIM']     = $row['ENTRFIM'];
        
        $query = "SELECT DISTINCT
                     FILIAL.CDFILIAL,
                     FILIAL.NMFILIAL,
                     NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AS CDFILCENTCMP,
                     FILCENT.NMFILIAL AS NMCENTCMP,
                     PARASUPRI.IDJUSSOLEXT,
                     PARASUPRI.IDBLOQALTSOLAUT,
                     PARASUPRI.IDAUDALTSOL,
                     PARASUPRI.IDULTIDTULTI,
                     PARASUPRI.IDULTCOMPSOL,
                     PARASUPRI.IDULTDESTSOL,
                     PARASUPRI.IDUTQTDEVOLU,
                     PARASUPRI.IDBAIXSOLDFS,
                     PARCMPSUP.IDUTPRODPREF,
                     PARCMPSUP.IDUTAPROVSOL,
                     PARCMPSUP.IDUTLOPERSOL,
                     PARCMPSUP.IDPERALTSAPR,
                     PARCMPSUP.IDIMPINCSOLAPR,
                     PARCMPSUP.NRORG
                  FROM
                     PARAMGERAL,
                     PARASUPRI,
                     PARCMPSUP,
                     ITEMSOLI,
                     SOLICITA,
                     FILIAL FILCENT,
                     (SELECT
                         FILIAL.CDFILIAL,
                         FILIAL.CDFILCENTCMP,
                         FILIAL.NMFILIAL,
                         FILIAL.NRORG
                      FROM
                         FILGRUPFILI,
                         FILIAL
                      WHERE
                        (FILIAL.IDCNTRRESC <> 'S')               AND
                        (FILIAL.CDREGIAO IS NOT NULL)            AND
                        (FILGRUPFILI.CDFILIAL = FILIAL.CDFILIAL) AND
                        (FILGRUPFILI.CDGRUPFILI = :P_CDGRUPFILI) AND
                        FILGRUPFILI.NRORG = FILIAL.NRORG         AND
                        FILIAL.NRORG = :NRORG                    AND
                  
                          (INSTR(:P_CDFILIAL, FILGRUPFILI.CDFILIAL) > 0 OR :P_CDFILIAL = 'T')
                  
                     UNION
                  
                     SELECT
                        CDFILIAL,
                        CDFILCENTCMP,
                        NMFILIAL,
                        NRORG
                     FROM
                        FILIAL
                     WHERE
                        (FILIAL.IDCNTRRESC <> 'S')     AND
                        (FILIAL.CDREGIAO IS NOT NULL)  AND
                        (:P_CDGRUPFILI IS NULL)        AND
                        FILIAL.NRORG = :NRORG          AND
                    
                          (INSTR(:P_CDFILIAL, FILIAL.CDFILIAL) > 0 OR :P_CDFILIAL = 'T')) FILIAL
                   WHERE
                       PARAMGERAL.NRORG =  PARASUPRI.NRORG AND
                       PARASUPRI.NRORG  = PARCMPSUP.NRORG AND
                       PARCMPSUP.NRORG  = SOLICITA.NRORG AND
                       SOLICITA.NRORG   = FILCENT.NRORG AND
                       FILCENT.NRORG   = FILIAL.NRORG  AND
                       FILIAL.NRORG    = :NRORG AND
                       (PARASUPRI.CDFILIAL  = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)) AND
                  
                       (PARCMPSUP.CDFILIAL  = FILIAL.CDFILIAL)                           AND
                  
                       (NVL(FILIAL.CDFILCENTCMP,FILIAL.CDFILIAL) = FILCENT.CDFILIAL)     AND
                  
                       (SOLICITA.CDFILSOLI  = FILIAL.CDFILIAL)                           AND
                  
                       ((SOLICITA.DTSOLCMP BETWEEN :P_DTSOLINI AND :P_DTSOLFIM) OR(:P_TSOLICITA = 'T'))  AND
                  
                       (INSTR(:P_COMPRADOR, SOLICITA.CDOPERADOR) <> 0 OR :P_TCOMPRADOR = 'T') AND
                  
                       (INSTR(:P_DESTINO, SOLICITA.CDDESTSOLI) <> 0 OR   :P_TDESTINO = 'T') AND
                     (ITEMSOLI.CDFILSOLI = SOLICITA.CDFILSOLI)  AND
                     (ITEMSOLI.CDFILLAN  = SOLICITA.CDFILLANCA) AND
                     (ITEMSOLI.NRSOLICMP = SOLICITA.NRSOLICMP)  AND
                     (ITEMSOLI.DTPROGENTR BETWEEN :P_ENTRINI AND :P_ENTRFIM)  AND
                     (:P_TCDGERENTE = 'T' OR
                     EXISTS (SELECT 1
                               FROM APROVSOLPRODGER A, DESTSOLGERE D
                              WHERE A.CDFILLANCA = SOLICITA.CDFILLANCA
                                AND A.CDFILSOLI  = SOLICITA.CDFILSOLI
                                AND A.NRSOLICMP  = SOLICITA.NRSOLICMP
                                AND A.CDFILSOLI  = D.CDFILIAL
                                AND A.CDOPERADOR = D.CDOPERADOR
                                AND D.CDDESTSOLI = SOLICITA.CDDESTSOLI
                                AND D.CDOPERADOR = :P_CDGERENTE
                                AND D.NRORG      =  A.NRORG
                                AND A.NRORG      = SOLICITA.NRORG
                  
                                  AND ((INSTR(:P_STATUS, (NVL(A.IDAPROVPRGER, 'X'))) > 0) OR (:P_STATUS =  'T'))
                                )) AND
                     (PARAMGERAL.IDRESTRFILIOPER = 'N' OR
                     EXISTS (SELECT 1
                               FROM FILIOPER
                              WHERE FILIOPER.CDOPERADOR = :P_CDOPERADOR
                                AND FILIOPER.CDFILIAL   =  SOLICITA.CDFILSOLI
                                AND FILIOPER.NRORG = :NRORG))
                  ORDER BY
                     FILIAL.CDFILIAL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
        
    }
    
    public function selectFiltroServ ($row){
        $param = array();
        $param['P_CDFILIAL'] = $row['CDFILIAL'];
        $param['P_DTSOLINI'] = $row['DTSOLINI'];
        $param['P_DTSOLFIM'] = $row['DTSOLFIM'];
        $param['P_TSOLICITA'] = $row['TSOLICITA'];
        $param['P_COMPRADOR'] = $row['COMPRADOR'];
        $param['P_TCOMPRADOR'] = $row['TCOMPRADOR'];
        $param['P_DESTINO'] = $row['CDDESTSOLI'];
        $param['P_TDESTINO'] = $row['TCDDESTSOLI'];
        $param['P_TCDGERENTE'] = $row['TCDGERENTE'];
        $param['P_CDGERENTE'] = $row['CDGERENTE'];
        $param['P_STATUS'] = $row['APROVACAO'];
        $param['P_CDOPERADOR'] = $row['CDOPERADOR'];
        $param['NRORG'] = $this->environment->getNrOrg();
        
        $param['P_TENTREGA'] = $row['TENTREGA'];
        $param['P_ENTRINI'] = $row['ENTRINI'];
        $param['P_ENTRFIM'] = $row['ENTRFIM'];
        
        $query = "SELECT
                     SOLICITA.CDFILSOLI,
                     SOLICITA.NRSOLICMP,
                     SOLICITA.CDFILLANCA,
                     SOLICITA.DTSOLCMP,
                     OPERADOR.NMOPERADOR,
                     DESTSOL.NMDESTSOLI,
                     DESTSOL.CDDESTSOLI,
                     SOLICITA.IDTIPLAN,
                     SOLICITA.IDSOLEXTRA,
                     SOLICITA.IDBAIXASOL,
                     SOLICITA.CDOPERLANC,
                     OPERLANC.NMOPERADOR NMOPERLANC,
                     SOLICITA.IDIMPSOLSAC,
                     SOLICITA.NRNECECMPSOL
                  FROM
                     DESTSOL,
                     OPERADOR OPERLANC,
                     OPERADOR,
                     SOLICITA,
                     PARAMGERAL
                  WHERE
                    PARAMGERAL.NRORG = SOLICITA.NRORG   AND
                    SOLICITA.NRORG  = OPERADOR.NRORG    AND
                    OPERADOR.NRORG  = OPERLANC.NRORG    AND
                    OPERLANC.NRORG   = DESTSOL.NRORG    AND
                    DESTSOL.NRORG    = :NRORG           AND
                    (SOLICITA.CDFILSOLI  = :P_CDFILIAL) AND
                    ((SOLICITA.DTSOLCMP BETWEEN :P_DTSOLINI AND :P_DTSOLFIM) OR
                     (:P_TSOLICITA = 'T'))  AND
                    (SOLICITA.CDFILSOLI  = DESTSOL.CDFILIAL (+))    AND
                    (SOLICITA.CDDESTSOLI = DESTSOL.CDDESTSOLI (+))  AND
                    (SOLICITA.CDOPERADOR = OPERADOR.CDOPERADOR(+))  AND
                    (INSTR(:P_COMPRADOR, SOLICITA.CDOPERADOR) <> 0  OR
                     :P_TCOMPRADOR = 'T')                           AND
                    (INSTR(:P_DESTINO, SOLICITA.CDDESTSOLI) <> 0    OR
                     :P_TDESTINO   = 'T')                           AND
                    (SOLICITA.CDOPERLANC = OPERLANC.CDOPERADOR(+))  AND
                    EXISTS
                    (SELECT
                        '1'
                     FROM
                        ITEMSOLI
                     WHERE
                        ITEMSOLI.CDFILSOLI = SOLICITA.CDFILSOLI  AND
                        ITEMSOLI.CDFILLAN  = SOLICITA.CDFILLANCA AND
                        ITEMSOLI.NRSOLICMP = SOLICITA.NRSOLICMP  AND
                        ITEMSOLI.NRORG = :NRORG                  AND
                        ((ITEMSOLI.DTPROGENTR BETWEEN :P_ENTRINI AND :P_ENTRFIM)
                        OR (:P_TENTREGA = 'T')))                 AND
                     (:P_TCDGERENTE = 'T' OR
                     EXISTS (SELECT 1
                               FROM APROVSOLPRODGER A, DESTSOLGERE D
                              WHERE A.CDFILLANCA = SOLICITA.CDFILLANCA
                                AND A.CDFILSOLI  = SOLICITA.CDFILSOLI
                                AND A.NRSOLICMP  = SOLICITA.NRSOLICMP
                                AND A.CDFILSOLI  = D.CDFILIAL
                                AND A.CDOPERADOR = D.CDOPERADOR
                                AND D.CDDESTSOLI = SOLICITA.CDDESTSOLI
                                AND D.CDOPERADOR = :P_CDGERENTE
                                AND D.NRORG      =  A.NRORG
                                AND A.NRORG      = SOLICITA.NRORG
                  
                                  AND ((INSTR(:P_STATUS, (NVL(A.IDAPROVPRGER, 'X'))) > 0) OR (:P_STATUS =  'T'))
                                )) AND
                     (PARAMGERAL.IDRESTRFILIOPER = 'N' OR
                     EXISTS (SELECT 1
                               FROM FILIOPER
                              WHERE FILIOPER.CDOPERADOR = :P_CDOPERADOR
                                AND FILIOPER.CDFILIAL   =  SOLICITA.CDFILSOLI
                                AND FILIOPER.NRORG = :NRORG))
                  ORDER BY
                     SOLICITA.DTSOLCMP DESC,
                     SOLICITA.NRSOLICMP DESC";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function selectSoliciFilial ($row){
        $param = array();
        $param['P_CDFILIAL'] = $row['CDFILIAL'];
        $param['P_DTSOLINI'] = $row['DTSOLINI'];
        $param['P_DTSOLFIM'] = $row['DTSOLFIM'];
        $param['P_TSOLICITA'] = $row['TSOLICITA'];
        $param['P_COMPRADOR'] = $row['COMPRADOR'];
        $param['P_TCOMPRADOR'] = $row['TCOMPRADOR'];
        $param['P_DESTINO'] = $row['CDDESTSOLI'];
        $param['P_TDESTINO'] = $row['TCDDESTSOLI'];
        $param['P_TCDGERENTE'] = $row['TCDGERENTE'];
        $param['P_CDGERENTE'] = $row['CDGERENTE'];
        $param['P_STATUS'] = $row['APROVACAO'];
        $param['P_CDOPERADOR'] = $row['CDOPERADOR'];
        $param['NRORG'] = $this->environment->getNrOrg();
        
        $param['P_TENTREGA'] = $row['TENTREGA'];
        $param['P_ENTRINI'] = $row['ENTRINI'];
        $param['P_ENTRFIM'] = $row['ENTRFIM'];
        
        $query = "SELECT
                   SOLICITA.CDFILSOLI,
                   SOLICITA.NRSOLICMP,
                   SOLICITA.CDFILLANCA,
                   SOLICITA.DTSOLCMP,
                   OPERADOR.NMOPERADOR,
                   DESTSOL.NMDESTSOLI,
                   DESTSOL.CDDESTSOLI,
                   SOLICITA.IDTIPLAN,
                   SOLICITA.IDSOLEXTRA,
                   SOLICITA.IDBAIXASOL,
                   SOLICITA.CDOPERLANC,
                   OPERLANC.NMOPERADOR NMOPERLANC,
                   SOLICITA.IDIMPSOLSAC,
                   SOLICITA.NRNECECMPSOL
                FROM
                   DESTSOL,
                   OPERADOR OPERLANC,
                   OPERADOR,
                   SOLICITA,
                   PARAMGERAL
                WHERE
                    PARAMGERAL.NRORG = SOLICITA.NRORG   AND
                    SOLICITA.NRORG  = OPERADOR.NRORG    AND
                    OPERADOR.NRORG  = OPERLANC.NRORG    AND
                    OPERLANC.NRORG   = DESTSOL.NRORG    AND
                    DESTSOL.NRORG    = :NRORG           AND
                  (SOLICITA.CDFILSOLI  = :P_CDFILIAL) AND
                  ((SOLICITA.DTSOLCMP BETWEEN :P_DTSOLINI AND :P_DTSOLFIM) OR
                   (:P_TSOLICITA = 'T'))  AND
                  (SOLICITA.CDFILSOLI  = DESTSOL.CDFILIAL (+)) AND
                  (SOLICITA.CDDESTSOLI = DESTSOL.CDDESTSOLI (+)) AND
                  (SOLICITA.CDOPERADOR = OPERADOR.CDOPERADOR(+)) AND
                  (INSTR(:P_COMPRADOR, SOLICITA.CDOPERADOR) <> 0 OR
                   :P_TCOMPRADOR = 'T') AND
                  (INSTR(:P_DESTINO, SOLICITA.CDDESTSOLI) <> 0 OR
                   :P_TDESTINO   = 'T') AND
                  (SOLICITA.CDOPERLANC = OPERLANC.CDOPERADOR(+)) AND
                  EXISTS
                  (SELECT
                      '1'
                   FROM
                      ITEMSOLI
                   WHERE
                      ITEMSOLI.CDFILSOLI = SOLICITA.CDFILSOLI  AND
                      ITEMSOLI.CDFILLAN  = SOLICITA.CDFILLANCA AND
                      ITEMSOLI.NRSOLICMP = SOLICITA.NRSOLICMP  AND
                      ITEMSOLI.NRORG     = :NRORG              AND
                      ((ITEMSOLI.DTPROGENTR BETWEEN :P_ENTRINI AND :P_ENTRFIM)
                      OR (:P_TENTREGA = 'T'))) AND
                   (:P_TCDGERENTE = 'T' OR
                   EXISTS (SELECT 1
                             FROM APROVSOLPRODGER A, DESTSOLGERE D
                            WHERE A.CDFILLANCA = SOLICITA.CDFILLANCA
                              AND A.CDFILSOLI  = SOLICITA.CDFILSOLI
                              AND A.NRSOLICMP  = SOLICITA.NRSOLICMP
                              AND A.CDFILSOLI  = D.CDFILIAL
                              AND A.CDOPERADOR = D.CDOPERADOR
                              AND D.CDDESTSOLI = SOLICITA.CDDESTSOLI
                              AND D.CDOPERADOR = :P_CDGERENTE
                              AND D.NRORG      =  A.NRORG
                              AND A.NRORG      = SOLICITA.NRORG
                              AND ((INSTR(:P_STATUS, (NVL(A.IDAPROVPRGER, 'X'))) > 0) OR (:P_STATUS =  'T'))
                              )) AND
                   (PARAMGERAL.IDRESTRFILIOPER = 'N' OR
                   EXISTS (SELECT 1
                             FROM FILIOPER
                            WHERE FILIOPER.CDOPERADOR = :P_CDOPERADOR
                              AND FILIOPER.CDFILIAL   =  SOLICITA.CDFILSOLI
                              AND FILIOPER.NRORG      = :NRORG))
                ORDER BY
                   SOLICITA.DTSOLCMP DESC,
                   SOLICITA.NRSOLICMP DESC";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getSolicitacao ($row){
        $param = array();
        $param['P_CDFILLANCA']  = $row['CDFILLANCA'];
        $param['P_CDFILSOLI']   = $row['CDFILSOLI'];
        $param['P_NRSOLICMP']   = $row['NRSOLICMP'];
        $param['NRORG']         = $this->environment->getNrOrg();
        $query = "SELECT
                     SOLICITA.CDFILSOLI,
                     SOLICITA.CDFILLANCA,
                     SOLICITA.NRSOLICMP,
                     TO_CHAR(SOLICITA.DTSOLCMP, 'dd/mm/yyyy') AS DTSOLCMP,
                     SOLICITA.IDTIPLAN,
                     SOLICITA.CDFILCENT,
                     SOLICITA.CDOPERADOR,
                     OPERADOR.NMOPERADOR,
                     SOLICITA.CDDESTSOLI,
                     DESTSOL.NMDESTSOLI,
                     FILIAL.NMFILIAL    AS NMFILIAL_LANCAMENTO,
                     FILISOLI.NMFILIAL  AS NMFILIAL_SOLICITANTE,
                     SOLICITA.IDSOLEXTRA,
                     SOLICITA.CDOPERLANC,
                     ' ' IDINSVRCMPANT,
                     SOLICITA.NRNECECMPSOL,
                     SOLICITA.NRORG
                  FROM
                     SOLICITA,
                     DESTSOL,
                     FILIAL,
                     FILIAL FILISOLI,
                     OPERADOR
                  WHERE
                    OPERADOR.NRORG       = FILISOLI.NRORG          AND
                    FILISOLI.NRORG       = FILIAL.NRORG            AND
                    FILIAL.NRORG         = DESTSOL.NRORG           AND
                    DESTSOL.NRORG        = SOLICITA.NRORG          AND
                    SOLICITA.NRORG       = :NRORG                  AND
                    (CDFILSOLI           = :P_CDFILSOLI)           AND
                    (CDFILLANCA          = :P_CDFILLANCA)          AND
                    (NRSOLICMP           = :P_NRSOLICMP)           AND
                    (SOLICITA.CDFILSOLI  = DESTSOL.CDFILIAL (+))   AND
                    (SOLICITA.CDDESTSOLI = DESTSOL.CDDESTSOLI (+)) AND
                    (FILIAL.CDFILIAL     = SOLICITA.CDFILLANCA)    AND
                    (FILISOLI.CDFILIAL   = SOLICITA.CDFILSOLI )    AND
                    (SOLICITA.CDOPERADOR = OPERADOR.CDOPERADOR(+))";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getValorSolicitacao ($row){
        $param = array();
        $param['P_CDFILLAN']    = $row['CDFILLANCA'];
        $param['P_CDFILSOLI']   = $row['CDFILSOLI'];
        $param['P_NRSOLICMP']   = $row['NRSOLICMP'];
        $param['P_NRTIPOCUSTO'] = $row['NRTIPOCUSTO'];
        $param['NRORG']         = $this->environment->getNrOrg();
        $query = "SELECT SUM((CUSTO_PRODUTO(:P_CDFILSOLI, 
                    PRODUTO.CDPRODESTO, 
                    :P_NRTIPOCUSTO, 
                    ITEMSOLI.DTPROGENTR, 
                    SYSDATE)*(NVL(PRODUTO.VRFATOCONV, 1)))*(NVL(ITEMSOLI.QTSOLIPROD, 0))) AS VRCUSTOTOTAL,
                    MAX(IDSOLIAPROV) IDSOLIAPROV
                  FROM ITEMSOLI, PRODUTO
                  WHERE
                        PRODUTO.NRORG = ITEMSOLI.NRORG
                    AND ITEMSOLI.NRORG = :NRORG
                    AND CDFILSOLI =:P_CDFILSOLI 
                    AND CDFILLAN  =:P_CDFILLAN
                    AND NRSOLICMP =:P_NRSOLICMP
                    AND ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODUTO";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function deleteSolicitacao ($row)
    {
        $query = "DELETE FROM SOLICITA 
                   WHERE CDFILSOLI  = :OLD_CDFILSOLI
                     AND CDFILLANCA = :OLD_CDFILLANCA
                     AND NRSOLICMP  = :OLD_NRSOLICMP
                     AND NRORG      = :NRORG";
        
        $param = array(
            'OLD_CDFILSOLI' => $row['CDFILSOLI'],
            'OLD_CDFILLANCA'=> $row['CDFILLANCA'],
            'OLD_NRSOLICMP' => $row['NRSOLICMP'],
            'NRORG'         => $this->environment->getNrOrg()
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function deleteSolicitacaoGer ($row)
    {
        $query = "DELETE FROM APROVSOLPRODGER
                   WHERE CDFILLANCA = :OLD_CDFILLANCA
                     AND CDFILSOLI  = :OLD_CDFILSOLI
                     AND NRSOLICMP  = :OLD_NRSOLICMP
                     AND NRORG      = :NRORG";
                     
        $param = array(
            'OLD_CDFILSOLI' => $row['CDFILSOLI'],
            'OLD_CDFILLANCA'=> $row['CDFILLANCA'],
            'OLD_NRSOLICMP' => $row['NRSOLICMP'],
            'NRORG'         => $this->environment->getNrOrg()
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function selectProdutos ($row, $oneRecord){
        $param = array();
        $param['P_CDFILSOLI']   = $row['CDFILSOLI'];
        $param['P_CDFILLAN']    = $row['CDFILLANCA'];
        $param['P_NRSOLICMP']   = $row['NRSOLICMP'];
        $param['NRORG']         = $this->environment->getNrOrg();
        
        
        $query = "SELECT DISTINCT RTRIM(PRODUTO.NMPRODUTO) NMPRODUTO, PRODUTO.CDARVPROD, PRODUTO.CDPRODUTO, PRODUTO.NRORG
                  FROM     PRODUTO,
                          (SELECT
                             CDPRODUTO,
                             DTPROGENTR,
                             TRUNC(QTSOLIPROD,3) AS QTSOLIPROD
                           FROM
                             ITEMSOLI
                           WHERE
                            (CDFILSOLI = :P_CDFILSOLI ) AND
                            (CDFILLAN  = :P_CDFILLAN  ) AND
                            (NRSOLICMP = :P_NRSOLICMP ) AND
                            (NRORG     = :NRORG       )) ITEMSOLI
            
                  WHERE
                     (PRODUTO.CDPRODUTO = ITEMSOLI.CDPRODUTO) AND
                     (PRODUTO.NRORG = :NRORG)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function selectDetalhesProduto ($row){
        $param = array();
        $param['P_CDFILSOLI']  = $row['CDFILSOLI'];
        $param['P_CDFILLAN']   = $row['CDFILLANCA'];
        $param['P_NRSOLICMP']  = $row['NRSOLICMP'];
        $param['P_CDPRODUTO']  = $row['CDPRODUTO'];
        $param['NRORG']        = $this->environment->getNrOrg();
        
        $query = "SELECT  RTRIM(PRODUTO.NMPRODUTO) || '||' || PRODUTO.CDARVPROD NMPRODUTO,
                           DTPROGENTR, QTSOLIPROD, IDSOLIAPROV, CDFILSOLI, CDFILLAN, CDFILLAN AS CDFILLANCA, NRSOLICMP, 
                           ITEMSOLI.CDPRODORIG, ITEMSOLI.QTSOLIORIG, NRORG
                  FROM     PRODUTO,
                          (SELECT
                             CDPRODUTO,
                             DTPROGENTR,
                             IDSOLIAPROV,
                             CDFILSOLI,
                             CDFILLAN,
                             CDFILLAN AS CDFILLANCA,
                             NRSOLICMP,
                             CDPRODORIG,
                             QTSOLIORIG,
                             TRUNC(QTSOLIPROD,3) AS QTSOLIPROD
                           FROM
                             ITEMSOLI
                           WHERE
                            (CDFILSOLI = :P_CDFILSOLI ) AND
                            (CDFILLAN  = :P_CDFILLAN  ) AND
                            (NRSOLICMP = :P_NRSOLICMP ) AND
                            (CDPRODUTO = :P_CDPRODUTO )) ITEMSOLI
                  WHERE
                     (PRODUTO.CDPRODUTO = ITEMSOLI.CDPRODUTO) AND
                     (PRODUTO.NRORG = :NRORG)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function decisionProdutos ($row){
        $param = array();
        $param['P_CDFILSOLI']   = $row['CDFILSOLI'];
        $param['P_CDFILLAN']    = $row['CDFILLANCA'];
        $param['P_NRSOLICMP']   = $row['NRSOLICMP'];
        $param['NRORG']         = $this->environment->getNrOrg();
        
        $query = "SELECT
                     COUNT(*) AS TOTAL
                  FROM
                     ITEMSOLI
                  WHERE
                     (CDFILSOLI  = :P_CDFILSOLI) AND
                     (CDFILLAN   = :P_CDFILLAN ) AND
                     (NRSOLICMP  = :P_NRSOLICMP) AND
                     (NRORG      = :NRORG      )";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getJustSolicitacao ($row){
        $param = array();
        $param['P_CDFILSOLI']   = $row['CDFILSOLI'];
        $param['P_CDFILLANCA']  = $row['CDFILLANCA'];
        $param['P_NRSOLICMP']   = $row['NRSOLICMP'];
        $param['NRORG']         = $this->environment->getNrOrg();
        
        $query = "SELECT
                    MOVSOLEXT.CDFILSOLEXT,
                    MOVSOLEXT.CDFILLANCEXT,
                    MOVSOLEXT.NRSOLIMOVEXT,
                    MOVSOLEXT.DTMOVSOLEXT,
                    MOVSOLEXT.CDOPERADOR,
                    MOVSOLEXT.DSJUSTSOLEXT,
                    MOVSOLEXT.NRORG
                  FROM
                    MOVSOLEXT
                  WHERE
                    MOVSOLEXT.CDFILSOLEXT  = :P_CDFILSOLI  AND
                    MOVSOLEXT.CDFILLANCEXT = :P_CDFILLANCA AND
                    MOVSOLEXT.NRSOLIMOVEXT = :P_NRSOLICMP  AND
                    MOVSOLEXT.NRORG = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function insJustSoli ($row, $NRSOLICMP){
        $param = array();
        $param['CDFILSOLEXT']   = $row['CDFILSOLI'];
        $param['CDFILLANCEXT']  = $row['CDFILLANCA'];
        $param['NRSOLIMOVEXT']  = $NRSOLICMP;
        $param['DTMOVSOLEXT']   = $row['DTSOLCMP'];
        $param['CDOPERADOR']    = $row['CDOPERLANC'];
        $param['DSJUSTSOLEXT']  = $row['DSJUSTSOL'];
        $param['NRORG']         = $this->environment->getNrOrg();
        
        $query = "INSERT INTO MOVSOLEXT
                    (CDFILSOLEXT, CDFILLANCEXT, NRSOLIMOVEXT, DTMOVSOLEXT, CDOPERADOR, DSJUSTSOLEXT, NRORG)
                 VALUES
                    (:CDFILSOLEXT, :CDFILLANCEXT, :NRSOLIMOVEXT, :DTMOVSOLEXT, :CDOPERADOR, :DSJUSTSOLEXT, :NRORG)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function updJustSoli ($row){
        $param = array();
        $param['CDFILSOLEXT']       = $row['CDFILSOLI'];
        $param['CDFILLANCEXT']      = $row['CDFILLANCA'];
        $param['NRSOLIMOVEXT']      = $row['NRSOLICMP'];;
        $param['DTMOVSOLEXT']       = $row['DTSOLCMP'];
        $param['CDOPERADOR']        = $row['CDOPERLANC'];
        $param['DSJUSTSOLEXT']      = $row['DSJUSTSOL'];
        $param['OLD_CDFILSOLEXT']   = $row['CDFILSOLI'];
        $param['OLD_CDFILLANCEXT']  = $row['CDFILLANCA'];
        $param['OLD_NRSOLIMOVEXT']  = $row['NRSOLICMP'];
        $param['OLD_DTMOVSOLEXT']   = $row['DTSOLCMP'];
        $param['NRORG']             = $this->environment->getNrOrg();
        
        $query = "UPDATE MOVSOLEXT
                SET
                  CDFILSOLEXT  = :CDFILSOLEXT,
                  CDFILLANCEXT = :CDFILLANCEXT,
                  NRSOLIMOVEXT = :NRSOLIMOVEXT,
                  DTMOVSOLEXT  = :DTMOVSOLEXT,
                  CDOPERADOR   = :CDOPERADOR,
                  DSJUSTSOLEXT = :DSJUSTSOLEXT
                WHERE
                  CDFILSOLEXT  = :OLD_CDFILSOLEXT  AND
                  CDFILLANCEXT = :OLD_CDFILLANCEXT AND
                  NRSOLIMOVEXT = :OLD_NRSOLIMOVEXT AND
                  NRORG        = :NRORG            AND
                  DTMOVSOLEXT  = :OLD_DTMOVSOLEXT";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function GetItemSoli ($row){
        $param = array();
        $param['P_CDFILSOLI'] = $row['CDFILSOLI'];
        $param['P_CDFILLANC'] = $row['CDFILIAL'];
        $param['P_NRSOLICMP'] = $row['NRSOLICMP'];
        $param['NRORG']       = $this->environment->getNrOrg();
        
        $query = "SELECT
                    ITEMSOLI.CDFILSOLI,
                    ITEMSOLI.CDFILLAN,
                    ITEMSOLI.NRSOLICMP,
                    ITEMSOLI.CDPRODUTO,
                    PRODUTO.CDARVPROD,
                    ITEMSOLI.DTPROGENTR,
                    ITEMSOLI.DTUTILIZA,
                    ITEMSOLI.QTSOLIPROD,
                    ITEMSOLI.TXOBSITMSOL,
                    ITEMSOLI.IDBAIXITMSOL,
                    ITEMSOLI.QTTRANSFSOL,
                    ITEMSOLI.IDSOLIAPROV,
                    ITEMSOLI.QTVOLUITEMSO,
                    ITEMSOLI.CDPRODORIG,
                    ITEMSOLI.QTSOLIORIG,
                    ITEMSOLI.IDIMPITMSAC,
                    ITEMSOLI.DTULTATUITSOL,
                    ITEMSOLI.DTIMPORTASAC,
                    ITEMSOLI.NRSEQUCONSOL,
                    ITEMSOLI.VRCMPANTERIOR,
                    ITEMSOLI.DTCMPANTERIOR,
                    ITEMSOLI.IDINSVRCMPANT,
                    PRODUTO.NRORG
                  FROM
                      (SELECT
                        ITEMSOLI.CDFILSOLI,
                        ITEMSOLI.CDFILLAN,
                        ITEMSOLI.NRSOLICMP,
                        ITEMSOLI.CDPRODUTO,
                        ITEMSOLI.DTPROGENTR,
                        ITEMSOLI.DTUTILIZA,
                        TRUNC(ITEMSOLI.QTSOLIPROD,3) AS QTSOLIPROD,
                        ITEMSOLI.TXOBSITMSOL,
                        ITEMSOLI.IDBAIXITMSOL,
                        ITEMSOLI.QTTRANSFSOL,
                        ITEMSOLI.IDSOLIAPROV,
                        ITEMSOLI.QTVOLUITEMSO,
                        ITEMSOLI.CDPRODORIG,
                        ITEMSOLI.QTSOLIORIG,
                        ITEMSOLI.IDIMPITMSAC,
                        ITEMSOLI.DTULTATUITSOL,
                        ITEMSOLI.DTIMPORTASAC,
                        ITEMSOLI.NRSEQUCONSOL,
                        NVL(ITEMSOLI.VRCMPANTERIOR, 0) VRCMPANTERIOR,
                        ITEMSOLI.DTCMPANTERIOR,
                        ' ' IDINSVRCMPANT
                     FROM
                        ITEMSOLI
                     WHERE
                        ITEMSOLI.CDFILSOLI  = :P_CDFILSOLI AND
                        ITEMSOLI.CDFILLAN   = :P_CDFILLANC AND
                        ITEMSOLI.NRSOLICMP  = :P_NRSOLICMP AND
                        ITEMSOLI.NRORG      = :NRORG       ) ITEMSOLI,
                  
                     PRODUTO
                  WHERE
                    PRODUTO.CDPRODUTO   = ITEMSOLI.CDPRODUTO AND
                    PRODUTO.NRORG       = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getParamFilial ($row){
        $param = array();
        $param['P_CDFILIAL'] = $row['CDFILIAL'];
        $param['NRORG'] = $this->environment->getNrOrg();
        
        $query = "SELECT
                       CDFILIAL, 
                       IDUTILIZAINTSAC, 
                       IDBLPRCOMEST, 
                       NVL(IDUTMODPRODFIL, 'N') 
                       IDUTMODPRODFIL,
                       NRORG
                  FROM
                     PARAMFILIAL
                  WHERE
                     CDFILIAL = :P_CDFILIAL AND
                     NRORG    = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function queryProdCompra($row) {
        $param = array();
        $param['OLD_CDFILSOLI']     = $row['CDFILSOLI'];
        $param['OLD_CDFILLAN']      = $row['CDFILLANCA'];
        $param['OLD_NRSOLICMP']     = $row['NRSOLICMP'];
        $param['OLD_CDPRODUTO']     = $row['CDPRODUTO'];
        $param['OLD_DTPROGENTR']    = $row['DTPROGENTR'];
        $param['NRORG']             = $this->environment->getNrOrg();
        
        $query = "SELECT 
                    NRCOMPRA, NRORG
                 FROM 
                    PRODCOMPRA
                 WHERE 
                    CDFILSOLI   = :OLD_CDFILSOLI
                 AND CDFILLAN   = :OLD_CDFILLAN
                 AND NRSOLICMP  = :OLD_NRSOLICMP
                 AND CDPRODUTO  = :OLD_CDPRODUTO
                 AND DTPROGENTR = :OLD_DTPROGENTR
                 AND NRORG      = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function delForneced($row) {
        $param = array();
        $param['P_NRCOMPRA'] = $row['NRCOMPRA'];
        $param['NRORG']      = $this->environment->getNrOrg();
        
        $query = "DELETE
                 FROM    
                    FORPRCOMPRA
                 WHERE 
                    FORPRCOMPRA.NRCOMPRA = :P_NRCOMPRA AND
                    FORPRCOMPRA.NRORG    = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function delProdCompra($row) {
        $param = array();
        $param['OLD_NRCOMPRA']   = $row['NRCOMPRA'];
        $param['OLD_CDFILSOLI']  = $row['CDFILSOLI'];
        $param['OLD_CDFILLAN']   = $row['CDFILLANCA'];
        $param['OLD_NRSOLICMP']  = $row['NRSOLICMP'];
        $param['OLD_CDPRODUTO']  = $row['CDPRODUTO'];
        $param['OLD_DTPROGENTR'] = $row['DTPROGENTR'];
        $param['NRORG']          = $this->environment->getNrOrg();
        
        $query = "DELETE 
                 FROM 
                    PRODCOMPRA
                 WHERE
                    NRCOMPRA   = :OLD_NRCOMPRA   AND
                    CDFILSOLI  = :OLD_CDFILSOLI  AND
                    CDFILLAN   = :OLD_CDFILLAN   AND
                    NRSOLICMP  = :OLD_NRSOLICMP  AND
                    CDPRODUTO  = :OLD_CDPRODUTO  AND
                    DTPROGENTR = :OLD_DTPROGENTR AND
                    NRORG      = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function updCmp($row) {
        $param = array();
        $param['P_IDSITUCMP'] = $row['IDSITUCOMPRA'];
        $param['P_NRCOMPRA'] = $row['NRCOMPRA'];
        $param['NRORG'] = $this->environment->getNrOrg();
        
        $query = "UPDATE 
                    COMPRA
                 SET 
                    IDSITUCOMPRA = :P_IDSITUCMP
                 WHERE  
                    NRCOMPRA     = :P_NRCOMPRA AND
                    NRORG        = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function delItemSoli($row) {
        $param = array();
        $param['OLD_CDFILSOLI']  = $row['CDFILSOLI'];
        $param['OLD_CDFILLAN']   = $row['CDFILLANCA'];
        $param['OLD_NRSOLICMP']  = $row['NRSOLICMP'];
        $param['OLD_CDPRODUTO']  = $row['CDPRODUTO'];
        $param['OLD_DTPROGENTR'] = $row['DTPROGENTR'];
        $param['NRORG']          = $this->environment->getNrOrg();
        
        $query = "DELETE 
                 FROM 
                    ITEMSOLI
                 WHERE
                    CDFILSOLI   = :OLD_CDFILSOLI  AND
                    CDFILLAN    = :OLD_CDFILLAN   AND
                    NRSOLICMP   = :OLD_NRSOLICMP  AND
                    CDPRODUTO   = :OLD_CDPRODUTO  AND
                    DTPROGENTR  = :OLD_DTPROGENTR AND
                    NRORG       = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function queryExisteAprovSoliGer ($row){
        $param = array();
        $param['P_CDFILSOLI']  = $row['CDFILSOLI'];
        $param['P_CDFILLAN']   = $row['CDFILLANCA'];
        $param['P_NRSOLICMP']  = $row['NRSOLICMP'];
        $param['P_CDPRODUTO']  = $row['CDPRODUTO'];
        $param['NRORG']        = $this->environment->getNrOrg();
        
        $query = "SELECT 
                    CDFILSOLI, NRORG
                 FROM 
                    ITEMSOLI
                 WHERE 
                    CDFILSOLI       = :P_CDFILSOLI
                    AND CDFILLAN    = :P_CDFILLAN
                    AND NRSOLICMP   = :P_NRSOLICMP
                    AND CDPRODUTO   = :P_CDPRODUTO
                    AND NRORG       = :NRORG
                    AND QTSOLIPROD IS NOT NULL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function delAprovSoliGer($row) {
        $param = array();
        $param['P_CDFILSOLI']  = $row['CDFILSOLI'];
        $param['P_CDFILLAN']   = $row['CDFILLANCA'];
        $param['P_NRSOLICMP']  = $row['NRSOLICMP'];
        $param['P_CDPRODUTO']  = $row['CDPRODUTO'];
        $param['NRORG']        = $this->environment->getNrOrg();
        
        $query = "DELETE FROM APROVSOLPRODGER
                 WHERE CDFILSOLI    = :P_CDFILSOLI
                    AND CDFILLANCA  = :P_CDFILLAN
                    AND NRSOLICMP   = :P_NRSOLICMP
                    AND CDPRODUTO   = :P_CDPRODUTO
                    AND NRORG       = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function insItemSoli($row)
    {
        $param = array();
        $param['CDFILSOLI']       = $row['CDFILSOLI'];
        $param['CDFILLAN']        = $row['CDFILLANCA'];
        $param['NRSOLICMP']       = $row['NRSOLICMP'];
        $param['CDPRODUTO']       = $row['CDPRODUTO'];
        $param['DTPROGENTR']      = $row['DTPROGENTR'];
        $param['QTSOLIPROD']      = (float) $row['QTSOLIPROD'];
        $param['QTSOLIORIG']      = (float) $row['QTSOLIPROD'];
        $param['QTVOLUITEMSO']    = (float) $row['QTVOLUITEMSO'];
        $param['TXOBSITMSOL']     = $row['TXOBSITMSOL'];
        $param['DTCMPANTERIOR']   = $row['DTCMPANTERIOR'];
        $param['IDSOLIAPROV']     = $row['IDSOLIAPROV'];
        $param['NRORG']           = $this->environment->getNrOrg();
        
        $query = "INSERT INTO ITEMSOLI(
                    CDFILSOLI,
                    CDPRODUTO,
                    NRSOLICMP,
                    DTPROGENTR,
                    CDFILLAN,
                    QTSOLIPROD,
                    QTSOLIORIG,
                    QTVOLUITEMSO,
                    TXOBSITMSOL,
                    DTCMPANTERIOR,
                    IDSOLIAPROV,
                    NRORG
                ) VALUES (
                    :CDFILSOLI, 
                    :CDPRODUTO,
                    :NRSOLICMP,
                    TO_DATE(:DTPROGENTR, 'DD/MM/YYYY'),
                    :CDFILLAN,
                    :QTSOLIPROD,
                    :QTSOLIORIG,
                    :QTVOLUITEMSO,
                    :TXOBSITMSOL,
                    TO_DATE(:DTCMPANTERIOR, 'DD/MM/YYYY'),
                    :IDSOLIAPROV,
                    :NRORG
                )";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }

    public function queryFilialOL($row){
        $param = array();
        $param['P_CDFILIAL']    = $row['CDFILIAL'];
        $param['P_CDPRODUTO']   = $row['CDPRODUTO'];
        $param['NRORG']         = $this->environment->getNrOrg();
        
        $query = "SELECT
                    MAX(P.CDFILIOPELOG) CDFILIOPELOG
                 FROM
                    PARAMPRODSUP P
                 WHERE
                    P.CDFILIAL  = :P_CDFILIAL   AND
                    P.CDPRODUTO = :P_CDPRODUTO  AND
                    P.NRORG     = :NRORG        AND
                    P.CDFILIOPELOG IS NOT NULL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function updSolicitaInt($row) {
        $param = array();
        $param['OLD_CDFILSOLI'] = $row['CDFILSOLI'];
        $param['OLD_CDFILLAN']  = $row['CDFILLANCA'];
        $param['OLD_NRSOLICMP'] = $row['NRSOLICMP'];
        $param['NRORG']         = $this->environment->getNrOrg();
        
        $query = "UPDATE SOLICITA
                  SET
                    IDIMPSOLSAC = 'S'
                  WHERE
                    CDFILSOLI   = :OLD_CDFILSOLI AND
                    CDFILLANCA  = :OLD_CDFILLAN  AND
                    NRSOLICMP   = :OLD_NRSOLICMP AND
                    NRORG       = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function updItSoliInt($row) {
        $param = array();
        $param['CDFILSOLI']     = $row['CDFILSOLI'];
        $param['CDFILLAN']      = $row['CDFILLANCA'];
        $param['NRSOLICMP']     = $row['NRSOLICMP'];
        $param['CDPRODUTO']     = $row['CDPRODUTO'];
        $param['DTPROGENTR']    = $row['CDFILSOLI'];
        $param['DTUTILIZA']     = $row['DTPROGENTR'];
        $param['QTSOLIPROD']    = $row['QTSOLIPROD'];
        $param['TXOBSITMSOL']   = $row['TXOBSITMSOL'];
        $param['IDSOLIAPROV']   = $row['IDSOLIAPROV'];
        $param['QTVOLUITEMSO']  = $row['QTVOLUITEMSO'];
        $param['CDPRODORIG']    = $row['CDPRODORIG'];
        $param["CDOPERAPROV"]   = $row['CDOPERAPROV'];
        $param["DTHRAPROVSOL"]  = $row["DTHRAPROVSOL"];
        $param['NRORG']         = $this->environment->getNrOrg();
        
        if($row['QTSOLIORIG'] == null) {
            $param['QTSOLIORIG'] = 0;
        } else {
            $param['QTSOLIORIG'] = $row['QTSOLIORIG'];
        }
        
        $param['OLD_CDFILSOLI'] = $row['CDFILSOLI'];
        $param['OLD_CDFILLAN']  = $row['CDFILLANCA'];
        $param['OLD_NRSOLICMP'] = $row['NRSOLICMP'];
        $param['OLD_CDPRODUTO'] = $row['CDPRODUTO'];
        $param['OLD_DTPROGENTR']= $row['DTPROGENTR'];
        
        $query = "UPDATE ITEMSOLI
                    SET
                        CDFILSOLI       = :CDFILSOLI,
                        CDFILLAN        = :CDFILLAN,
                        NRSOLICMP       = :NRSOLICMP,
                        CDPRODUTO       = :CDPRODUTO,
                        DTPROGENTR      = :DTPROGENTR,
                        DTUTILIZA       = :DTUTILIZA,
                        QTSOLIPROD      = :QTSOLIPROD,
                        TXOBSITMSOL     = :TXOBSITMSOL,
                        IDSOLIAPROV     = :IDSOLIAPROV,
                        QTVOLUITEMSO    = :QTVOLUITEMSO,
                        CDPRODORIG      = :CDPRODORIG,
                        QTSOLIORIG      = :QTSOLIORIG,
                        IDIMPITMSAC     = 'S',
                        CDOPERAPROV     = :CDOPERAPROV,
                        DTHRAPROVSOL    = :DTHRAPROVSOL
                 WHERE
                          CDFILSOLI     = :OLD_CDFILSOLI
                    AND   CDFILLAN      = :OLD_CDFILLAN
                    AND   NRSOLICMP     = :OLD_NRSOLICMP
                    AND   CDPRODUTO     = :OLD_CDPRODUTO
                    AND   DTPROGENTR    = :OLD_DTPROGENTR
                    AND   NRORG         = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function updItemSoli($row) {
        $param = array();
        $param['CDFILSOLI']     = $row['CDFILSOLI'];
        $param['CDFILLAN']      = $row['CDFILLANCA'];
        $param['NRSOLICMP']     = $row['NRSOLICMP'];
        $param['CDPRODUTO']     = $row['CDPRODUTO'];
        $param['DTPROGENTR']    = $row['DTPROGENTR'];
        $param['IDSOLIAPROV']   = $row['IDSOLIAPROV'];
        $param['NRORG']         = $this->environment->getNrOrg();
        $param['QTSOLIPROD']    = self::parseFloat($row['QTSOLIPROD']);
        $param['TXOBSITMSOL']   = $row['TXOBSITMSOL'];
        $param['QTSOLIORIG']    = self::parseFloat($row['QTSOLIORIG']);
        $param['QTVOLUITEMSO']  = self::parseFloat($row['QTVOLUITEMSO']);
        
        $param['OLD_CDFILSOLI'] = $row['CDFILSOLI'];
        $param['OLD_CDFILLAN']  = $row['CDFILLANCA'];
        $param['OLD_NRSOLICMP'] = $row['NRSOLICMP'];
        $param['OLD_CDPRODUTO'] = $row['CDPRODUTO'];
        $param['OLD_DTPROGENTR']= $row['DTPROGENTR'];
        $param["CDOPERAPROV"]   = $row["CDOPERAPROV"];
        $param["DTHRAPROVSOL"]  = $row["DTHRAPROVSOL"];
        
        $replace = str_replace('/', '-', $param['DTPROGENTR']);
        $a = new \DateTime($replace);
        $a = $a->format('d/m/Y');
        
        $param['DTPROGENTR']    = $a;
        
        $replace = str_replace('/', '-', $param['OLD_DTPROGENTR']);
        $b = new \DateTime($replace);
        $b = $b->format('d/m/Y');
        
        $param['OLD_DTPROGENTR'] = $b;
        
        $query = "UPDATE ITEMSOLI
                  SET
                    CDFILSOLI     = :CDFILSOLI,
                    CDFILLAN      = :CDFILLAN,
                    NRSOLICMP     = :NRSOLICMP,
                    CDPRODUTO     = :CDPRODUTO,
                    DTPROGENTR    = TO_DATE(:DTPROGENTR, 'DD/MM/YYYY'),
                    QTSOLIPROD    = :QTSOLIPROD,
                    QTSOLIORIG    = :QTSOLIORIG,
                    QTVOLUITEMSO  = :QTVOLUITEMSO,
                    TXOBSITMSOL   = :TXOBSITMSOL,
                    IDSOLIAPROV   = :IDSOLIAPROV,
                    CDOPERAPROV   = :CDOPERAPROV,
                    DTHRAPROVSOL  = :DTHRAPROVSOL
                  WHERE
                    CDFILSOLI    = :OLD_CDFILSOLI AND
                    CDFILLAN     = :OLD_CDFILLAN  AND
                    NRSOLICMP    = :OLD_NRSOLICMP AND
                    CDPRODUTO    = :OLD_CDPRODUTO AND
                    NRORG        = :NRORG         AND
                    DTPROGENTR   = TO_DATE(:OLD_DTPROGENTR, 'DD/MM/YYYY')";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function insSolicita ($row, $NRSOLICMP)
    {
        $param = array();
        $param['CDFILSOLI']  = $row['CDFILIAL'];
        $param['CDFILLANCA'] = $row['CDFILLANCA'];
        $param['NRSOLICMP']  = $NRSOLICMP;
        $param['DTSOLCMP']   = $row['DTSOLCMP'];
        $param['IDTIPLAN']   = $row['IDTIPLAN'];
        $param['CDFILCENT']  = $row['CDFILCENT'];
        $param['CDOPERADOR'] = $row['CDOPERADOR'];
        $param['CDDESTSOLI'] = $row['CDDESTSOLI'];
        $param['IDSOLEXTRA'] = $row['IDSOLEXTRA'];
        $param['CDOPERLANC'] = $this->environment->getCdOperador();
        $param['NRORG']      = $this->environment->getNrOrg();
        
        $query = "INSERT INTO SOLICITA
                    (CDFILSOLI, 
                    CDFILLANCA, 
                    NRSOLICMP, 
                    DTSOLCMP, 
                    IDTIPLAN, 
                    CDFILCENT, 
                    CDOPERADOR, 
                    CDDESTSOLI, 
                    IDSOLEXTRA, 
                    CDOPERLANC,
                    NRORG)
                VALUES (:CDFILSOLI, 
                    :CDFILLANCA, 
                    :NRSOLICMP,
                    :DTSOLCMP, 
                    :IDTIPLAN, 
                    :CDFILCENT, 
                    :CDOPERADOR, 
                    :CDDESTSOLI, 
                    :IDSOLEXTRA, 
                    :CDOPERLANC,
                    :NRORG)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }    
    }
    
    public function listaProdutos($row){
        $param = array();
        $param['P_CDPRODUTOINI'] = $row['CDPRODINI'];
        $param['P_CDPRODUTOFIM'] = $row['CDPRODFIN'];
        $param['P_CDFILSOLI']    = $row['CDFILIAL'];
        $param['P_PRODCMP']      = $row['PRODCMP'];
        $param['NRORG']          = $this->environment->getNrOrg();

        $query = "SELECT
                       PRODUTO.CDPRODUTO,
                       PRODUTO.CDARVPROD,
                       CAST(RTRIM(PRODUTO.NMPRODUTO) AS VARCHAR(50))AS NMPRODUTO,
                       PRODUTO.SGUNIDADE,
                       PRODUTO.NRORG
                    FROM
                       PRODFILI,
                       PRODUTO
                    WHERE
                        PRODUTO.NRORG = PRODFILI.NRORG AND
                        PRODFILI.NRORG = :NRORG        AND
                        ((:P_PRODCMP = 'N' AND PRODUTO.CDPRODUTO = PRODUTO.CDPRODESTO)     OR
                        (:P_PRODCMP = 'S' AND PRODUTO.CDPRODESTO IS NOT NULL))  AND
                        (PRODUTO.CDARVPROD BETWEEN :P_CDPRODUTOINI AND RPAD(:P_CDPRODUTOFIM, 20, 'Z')) AND
                        (NVL(PRODFILI.IDPRODATIV, 'A') = 'A'   )     AND
                        (NVL(PRODUTO.IDPRODATIVPR,'S') = 'S'   )     AND 
                        (PRODFILI.CDPRODUTO(+) = PRODUTO.CDPRODUTO)  AND
                        (PRODFILI.CDFILIAL (+) = :P_CDFILSOLI)
                    ORDER BY 
                        PRODUTO.CDARVPROD";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getProdutoMin($row){
        $param = array();
        $param['P_CDFILIAL']    = $row['CDFILIAL'];
        $query = "SELECT
                   MIN(CDARVPROD) AS MIN_CDPRODUTO,
                   MAX(CDARVPROD) AS MAX_CDPRODUTO
                FROM
                   PRODFILI,
                   PRODUTO
                WHERE PRODUTO.CDPRODESTO = PRODUTO.CDPRODUTO
                  AND (NVL(PRODFILI.IDPRODATIV, 'A') = 'A'   )
                  AND (NVL(PRODUTO.IDPRODATIVPR,'S') = 'S'   )
                  AND (PRODFILI.CDPRODUTO(+) = PRODUTO.CDPRODUTO)
                  AND (PRODFILI.CDFILIAL (+) = :P_CDFILIAL)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function listaProdutosPorSolicitacao($row){
        $param = array();
        $param['P_CDPRODUTOINI']    = $row['CDPRODINI'];
        $param['P_CDPRODUTOFIM']    = $row['CDPRODFIN'];
        $param['P_CDFILSOLI']       = $row['CDFILIAL_SOL'];
        $param['P_CDFILIAL']        = $row['CDFILIAL_SOL'];
        $param['P_NRSOLICMPORIG']   = $row['NRSOLICMP_SOL'];
        $param['P_CDFILLANCA']      = $row['CDFILLANCA_SOL'];
        $param['P_PRODCMP']         = $row['PRODCMP'];
        $param['NRORG']             = $this->environment->getNrOrg();
        
        $query = "  SELECT DISTINCT
                        PRODUTO.CDPRODUTO,
                        PRODUTO.CDARVPROD,
                        RTRIM(PRODUTO.NMPRODUTO) AS NMPRODUTO,
                        PRODUTO.SGUNIDADE,
                        PRODUTO.NRORG
                    FROM
                        PRODFILI,
                        PRODUTO,
                        ITEMSOLI
                    WHERE
                        ITEMSOLI.NRORG = PRODUTO.NRORG AND
                        PRODUTO.NRORG = PRODFILI.NRORG AND
                        PRODFILI.NRORG = :NRORG        AND
                        (:P_PRODCMP = 'N' AND PRODUTO.CDPRODUTO = PRODUTO.CDPRODESTO     OR
                        :P_PRODCMP = 'S' AND PRODUTO.CDPRODESTO IS NOT NULL)  AND
                        (PRODUTO.CDARVPROD BETWEEN :P_CDPRODUTOINI AND :P_CDPRODUTOFIM) AND
                        (PRODUTO.CDPRODUTO  = ITEMSOLI.CDPRODUTO)  AND
                        (ITEMSOLI.NRSOLICMP = :P_NRSOLICMPORIG)    AND
                        (ITEMSOLI.CDFILLAN  = :P_CDFILLANCA)       AND
                        (ITEMSOLI.CDFILSOLI = :P_CDFILSOLI)        AND
                        (:P_NRSOLICMPORIG IS NOT NULL)             AND
                        (NVL(PRODFILI.IDPRODATIV, 'A') = 'A'   )     AND
                        (NVL(PRODUTO.IDPRODATIVPR,'S') = 'S'   )     AND  
                        (PRODFILI.CDPRODUTO(+) = PRODUTO.CDPRODUTO)  AND
                        (PRODFILI.CDFILIAL (+) = :P_CDFILIAL)
                    ORDER BY 
                        PRODUTO.CDARVPROD";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function updSolicita ($row)
    {
        $param = array(
            'DTSOLCMP'      => $row['DTSOLCMP'],
            'IDTIPLAN'      => $row['IDTIPLAN'],
            'CDFILCENT'     => $row['CDFILCENTCMP'],
            'CDOPERADOR'    => $row['CDOPERADOR'],
            'CDDESTSOLI'    => $row['CDDESTSOLI'],
            'IDSOLEXTRA'    => $row['IDSOLEXTRA'],
            'P_CDFILSOLI'   => $row['CDFILSOLI'],
            'P_CDFILLANCA'  => $row['CDFILLANCA'],
            'P_NRSOLICMP'   => $row['NRSOLICMP'],
            'NRORG'         => $this->environment->getNrOrg()
        );
        
        $query = "UPDATE SOLICITA
                     SET DTSOLCMP    = :DTSOLCMP,
                         IDTIPLAN    = :IDTIPLAN,  
                         CDFILCENT   = :CDFILCENT,  
                         CDOPERADOR  = :CDOPERADOR,
                         CDDESTSOLI  = :CDDESTSOLI,
                         IDSOLEXTRA  = :IDSOLEXTRA
                   WHERE CDFILSOLI   = :P_CDFILSOLI
                     AND CDFILLANCA  = :P_CDFILLANCA
                     AND NRSOLICMP   = :P_NRSOLICMP
                     AND NRORG       = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    //Processos tabela MOVOPSOL
    
    public function getMovoPSol ($row){
        $param = array();
        $param['P_CDFILSOLMOV'] = $row['CDFILSOLI'];
        $param['P_CDFILLANMOV'] = $row['CDFILLANCA'];
        $param['P_NRSOLIMOV']   = $row['NRSOLICMP'];
        $param['NRORG']         = $this->environment->getNrOrg();
        
        $query = "SELECT
                     LOG.CDFILSOLMOV,
                     LOG.CDFILLANMOV,
                     LOG.NRSOLIMOV,
                     LOG.DTOPERSOL,
                     LOG.CDOPERADOR,
                     LOG.DSJUSTOPER,
                     LOG.CDPRODUTO,
                     PRODUTO.CDARVPROD,
                     LOG.QTANTOPER,
                     LOG.QTDEPOPER,
                     LOG.DTENTROPER,
                     NMPRODUTO,
                     NMOPERADOR,
                     LOG.IDOPERAMOVS,
                     LOG.DSRDMUSUMOVS,
                     LOG.DSENDUSUMOVS,
                     LOG.DSNMUSUMOVS,
                     LOG.DSMETUSUMOVS,
                     LOG.NRORG
                  FROM
                     MOVOPSOL LOG,
                     PRODUTO,
                     OPERADOR
                  WHERE
                     OPERADOR.NRORG  = PRODUTO.NRORG        AND
                     PRODUTO.NRORG   = LOG.NRORG            AND
                     LOG.NRORG       = :NRORG               AND
                     (CDFILSOLMOV    = :P_CDFILSOLMOV)      AND
                     (CDFILLANMOV    = :P_CDFILLANMOV)      AND
                     (NRSOLIMOV      = :P_NRSOLIMOV  )      AND
                     (LOG.CDOPERADOR = OPERADOR.CDOPERADOR) AND
                     (LOG.CDPRODUTO  = PRODUTO.CDPRODUTO)
                  ORDER BY
                     LOG.DTENTROPER DESC";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function insMovoPSol($row)
    {
        $param = array(
            'CDFILSOLMOV' => $row['CDFILSOLI'],
            'CDFILLANMOV' => $row['CDFILLANCA'],
            'NRSOLIMOV'   => $row['NRSOLICMP'],
            'DSJUSTOPER'  => $row['DSJUSTOPER'],
            'CDPRODUTO'   => $row['CDPRODUTO'],
            'QTANTOPER'   => array_key_exists('OLD_QTSOLIPROD', $row) ? $row['OLD_QTSOLIPROD'] : 0,
            'QTDEPOPER'   => (double)$row['QTSOLIPROD'],
            'DTENTROPER'  => substr($row['DTPROGENTR'], 0, 10),
            'IDOPERAMOVS' => $row['IDOPERAMOVS'],
            'CDOPERADOR'  => $this->environment->getCdOperador(),
            'NRORG'       => $this->environment->getNrOrg()
        );
        
        $query = "INSERT INTO MOVOPSOL (CDFILSOLMOV, CDFILLANMOV, NRSOLIMOV, DTOPERSOL,
                                        CDOPERADOR, DSJUSTOPER, CDPRODUTO, QTANTOPER,
                                        QTDEPOPER, DTENTROPER, IDOPERAMOVS, NRORG)
                                VALUES (:CDFILSOLMOV, :CDFILLANMOV, :NRSOLIMOV, SYSDATE,
                                        :CDOPERADOR, :DSJUSTOPER, :CDPRODUTO, :QTANTOPER,
                                        :QTDEPOPER, TO_DATE(:DTENTROPER, 'DD/MM/YYYY'),
                                        :IDOPERAMOVS, :NRORG)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }    
    }
    
    public function updMovoPSol ($row, $movoPSol){
        $param = array();
        $param['OLD_CDFILSOLMOV'] = $movoPSol['CDFILSOLMOV'];
        $param['OLD_CDFILLANMOV'] = $movoPSol['CDFILLANMOV'];
        $param['OLD_NRSOLIMOV']   = $movoPSol['NRSOLIMOV'];
        $param['OLD_DTOPERSOL']   = $movoPSol['DTOPERSOL'];
        $param['OLD_CDPRODUTO']   = $movoPSol['CDPRODUTO'];
        $param['OLD_DTENTROPER']  = $movoPSol['DTENTROPER'];
        $param['OLD_IDOPERAMOVS'] = $movoPSol['IDOPERAMOVS'];
        
        $param['CDFILSOLMOV']     = $row['CDFILSOLI'];
        $param['CDFILLANMOV']     = $row['CDFILLANCA'];
        $param['NRSOLIMOV']       = $row['NRSOLICMP'];
        //$param['DTOPERSOL']     = new \DateTime();
        $param['CDOPERADOR']      = $row['CDOPERLANC'];
        $param['DSJUSTOPER']      = $row['DSJUSTOPER'];
        $param['CDPRODUTO']       = $row['CDPRODUTO'];
        $param['QTANTOPER']       = $row['OLD_QTSOLIPROD'];
        $param['QTDEPOPER']       = (float) $row['QTSOLIPROD'];
        $param['DTENTROPER']      = $row['DTPROGENTR'];
        $param['IDOPERAMOVS']     = 'A';
        $param['NRORG']           = $this->environment->getNrOrg();
        
        $query = "UPDATE MOVOPSOL
                    SET
                        CDFILSOLMOV     = :CDFILSOLMOV,
                        CDFILLANMOV     = :CDFILLANMOV,
                        NRSOLIMOV       = :NRSOLIMOV,
                        DTOPERSOL       = SYSDATE,
                        CDOPERADOR      = :CDOPERADOR,
                        DSJUSTOPER      = :DSJUSTOPER,
                        CDPRODUTO       = :CDPRODUTO,
                        QTANTOPER       = :QTANTOPER,
                        QTDEPOPER       = :QTDEPOPER,
                        DTENTROPER      = TO_DATE(:DTENTROPER, 'DD/MM/YYYY hh24:mi:ss'),   
                        IDOPERAMOVS     = :IDOPERAMOVS
                    WHERE
                        CDFILSOLMOV     = :OLD_CDFILSOLMOV AND
                        CDFILLANMOV     = :OLD_CDFILLANMOV AND
                        NRSOLIMOV       = :OLD_NRSOLIMOV AND
                        TRUNC(DTOPERSOL) = TO_DATE(:OLD_DTOPERSOL, 'DD/MM/YYYY hh24:mi:ss') AND
                        CDPRODUTO       = :OLD_CDPRODUTO AND
                        TRUNC(DTENTROPER) = TO_DATE(:OLD_DTENTROPER, 'DD/MM/YYYY hh24:mi:ss') AND    
                        IDOPERAMOVS     = :OLD_IDOPERAMOVS AND
                        NRORG           = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function updJustMovoPSol ($row){
        
        $param = array();
        $param['DSJUSTOPER']      = $row['DSJUSTOPER'];
        $param['CDFILSOLMOV']     = $row['CDFILSOLMOV'];
        $param['CDFILLANMOV']     = $row['CDFILLANMOV'];
        $param['NRSOLIMOV']       = $row['NRSOLIMOV'];
        //$param['DTOPERSOL']     = $row['DTOPERSOL'];
        $param['CDPRODUTO']       = $row['CDPRODUTO'];
        $param['NRORG']           = $this->environment->getNrOrg();
        
        $query = "UPDATE MOVOPSOL
                     SET DSJUSTOPER = :DSJUSTOPER
                   WHERE CDFILSOLMOV = :CDFILSOLMOV
                     AND CDFILLANMOV = :CDFILLANMOV
                     AND NRSOLIMOV = :NRSOLIMOV
                     AND CDPRODUTO = :CDPRODUTO
                     AND DTOPERSOL = SYSDATE
                     AND NRORG = :NRORG";
                    
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function delMovoPSol ($row){
        $param = array();
        $param['OLD_CDFILSOLMOV'] = $row['CDFILSOLMOV'];
        $param['OLD_CDFILLANMOV'] = $row['CDFILLANMOV'];
        $param['OLD_NRSOLIMOV']   = $row['NRSOLIMOV'];
        $param['OLD_DTOPERSOL']   = $row['DTOPERSOL'];
        $param['OLD_CDPRODUTO']   = $row['CDPRODUTO'];
        $param['OLD_DTENTROPER']  = $row['DTENTROPER'];
        $param['NRORG']           = $this->environment->getNrOrg();
        
        $query = "DELETE FROM 
                        MOVOPSOL
                    WHERE  
                        CDFILSOLMOV = :OLD_CDFILSOLMOV AND
                        CDFILLANMOV = :OLD_CDFILLANMOV AND
                        NRSOLIMOV   = :OLD_NRSOLIMOV   AND 
                        DTOPERSOL   = :OLD_DTOPERSOL   AND 
                        CDPRODUTO   = :OLD_CDPRODUTO   AND  
                        DTENTROPER  = :OLD_DTENTROPER  AND
                        NRORG       = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function updIdOperaMovS ($movoPSol){
        $param = array();
        
        $param['OLD_CDFILSOLMOV'] = $movoPSol['CDFILSOLMOV'];
        $param['OLD_CDFILLANMOV'] = $movoPSol['CDFILLANMOV'];
        $param['OLD_NRSOLIMOV']   = $movoPSol['NRSOLIMOV'];
        $param['OLD_DTOPERSOL']   = $movoPSol['DTOPERSOL'];
        $param['OLD_CDPRODUTO']   = $movoPSol['CDPRODUTO'];
        $param['OLD_DTENTROPER']  = $movoPSol['DTENTROPER'];
        $param['NRORG']           = $this->environment->getNrOrg();
        
        $query = "UPDATE 
                        MOVOPSOL 
                    SET
                        IDOPERAMOVS = 'E'
                    WHERE  
                        CDFILSOLMOV = :OLD_CDFILSOLMOV AND  
                        CDFILLANMOV = :OLD_CDFILLANMOV AND  
                        NRSOLIMOV = :OLD_NRSOLIMOV AND  
                        DTOPERSOL = :OLD_DTOPERSOL AND 
                        CDPRODUTO = :OLD_CDPRODUTO AND  
                        DTENTROPER = :OLD_DTENTROPER AND
                        NRORG     = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function validarInclusProdCompra($row) 
    {
        
        $param = array(
            'P_CDFILIAL' => $row['CDFILSOLI'],
            'NRORG'      => $this->environment->getNrOrg()
        );
        
        $query =   "SELECT  
                        FILIAL.CDFILIAL,
                        FILIAL.NMFILIAL,
                        PARASUPRI.IDJUSSOLEXT,
                        PARASUPRI.IDBLOQALTSOLAUT,
                        PARASUPRI.IDAUDALTSOL,
                        PARASUPRI.IDJUSSOLEXT,
                        PARASUPRI.IDULTIDTULTI,
                        PARASUPRI.IDULTCOMPSOL,
                        PARASUPRI.IDULTDESTSOL,
                        PARASUPRI.IDUTQTDEVOLU,
                        PARASUPRI.IDBAIXSOLDFS,
                        PARCMPSUP.IDUTPRODPREF,
                        PARCMPSUP.IDUTAPROVSOL,
                        PARCMPSUP.IDUTLOPERSOL,
                        NVL(PARCMPSUP.IDPERALTSAPR, 'NNN') IDPERALTSAPR,
                        NVL(PARCMPSUP.IDIMPINCSOLAPR, 'N') IDIMPINCSOLAPR,
                        PARASUPRI.IDAPROVITMSOLGR,
                        PARASUPRI.NRORG
                    FROM
                        PARASUPRI,
                        PARCMPSUP,
                        FILIAL
                    WHERE
                        FILIAL.NRORG    = PARCMPSUP.NRORG AND
                        PARCMPSUP.NRORG = PARASUPRI.NRORG AND
                        PARASUPRI.NRORG = :NRORG          AND
                        (PARASUPRI.CDFILIAL  = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)) AND
                        (PARCMPSUP.CDFILIAL  = FILIAL.CDFILIAL)                           AND
                        FILIAL.CDFILIAL = :P_CDFILIAL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function queryAutGrSoli() {
        $param = array();
        
        $param['P_CDOPERADOR'] = $this->environment->getCdOperador();
        $param['NRORG']        = $this->environment->getNrOrg();
        
        $query = "SELECT AUTGRUPOPSOLI.IDAUTLANCSOLI,
                         MIN(AUTGRUPOPSOLI.PEALTEQTDESOLI) PEALTEQTDESOLI,
                         AUTGRUPOPSOLI.NRORG
                    FROM AUTGRUPOPSOLI INNER JOIN OPERGRUPOP ON AUTGRUPOPSOLI.CDGRUPOPER = OPERGRUPOP.CDGRUPOPER
                                                            AND AUTGRUPOPSOLI.NRORG = OPERGRUPOP.NRORG
                   WHERE AUTGRUPOPSOLI.IDAUTLANCSOLI IS NOT NULL
                     AND OPERGRUPOP.CDOPERADOR = :P_CDOPERADOR
                     AND OPERGRUPOP.NRORG = AUTGRUPOPSOLI.NRORG
                     AND AUTGRUPOPSOLI.NRORG = :NRORG
                   GROUP BY AUTGRUPOPSOLI.IDAUTLANCSOLI, AUTGRUPOPSOLI.NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getIdTiPlan ($row){
        $param = array();
        $param['NRSOLICMP']  = $row['NRSOLICMP'];
        $param['CDFILSOLI']  = $row['CDFILSOLI'];
        $param['CDFILLANCA'] = $row['CDFILLANCA'];
        $param['NRORG']      = $this->environment->getNrOrg();
        
        $query = "SELECT IDTIPLAN FROM SOLICITA WHERE NRSOLICMP = :NRSOLICMP AND CDFILSOLI = :CDFILSOLI AND CDFILLANCA = :CDFILLANCA AND NRORG = :NRORG";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getTipoCusto ($row){
        $param = array();
        $param['P_CDFILIAL'] = $row['CDFILIAL'];
        $param['P_NRORG'] = $this->environment->getNrOrg();
        
        $query = "SELECT CDTIPOCUSTO, NMTIPOCUSTO
                    FROM TIPOCUSTO, PARAMPLANEJA
                   WHERE TIPOCUSTO.CDTIPOCUSTO = PARAMPLANEJA.CDTIPOCUSTODESC
                     AND TIPOCUSTO.NRORG = PARAMPLANEJA.NRORG
                     AND PARAMPLANEJA.NRORG = :P_NRORG
                     AND PARAMPLANEJA.CDFILIAL = :P_CDFILIAL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function selectProdutosGrid ($row){
        $param = array();
        $param['NRORG']       = $this->environment->getNrOrg();
        $param['P_NRSOLICMP']   = $row['NRSOLICMP'];
        $param['P_CDFILSOLI']   = $row['CDFILSOLI'];
        $param['P_CDFILLAN']  =   $row['CDFILLANCA'];
        $param['P_NRTIPOCUSTO'] = $row['NRTIPOCUSTO'];
        $param['P_ONE_RECORD'] = 'N';
        $query = "SELECT RTRIM(PRODUTO.NMPRODUTO) NMPRODUTO, PRODUTO.CDARVPROD, PRODUTO.CDPRODUTO, PRODUTO.SGUNIDADE, PRODUTO.NRORG, ITEMSOLI.IDSOLIAPROV, SUM(ITEMSOLI.QTSOLIPROD) QTTOTAL, ( CUSTO_PRODUTO(:P_CDFILSOLI, PRODUTO.CDPRODESTO, :P_NRTIPOCUSTO, ITEMSOLI.DTPROGENTR, TRUNC(SYSDATE)) * ( NVL(PRODUTO.VRFATOCONV , 1) ) ) VRUNIT, SUM((CUSTO_PRODUTO(:P_CDFILSOLI, PRODUTO.CDPRODESTO, :P_NRTIPOCUSTO, ITEMSOLI.DTPROGENTR, TRUNC(SYSDATE)) *(NVL(PRODUTO.VRFATOCONV , 1))) *(NVL(ITEMSOLI.QTSOLIPROD, 0))) AS VRCUSTO FROM PRODUTO, ( SELECT CDPRODUTO, DTPROGENTR, IDSOLIAPROV, TRUNC(QTSOLIPROD, 3) AS QTSOLIPROD FROM ITEMSOLI WHERE ( CDFILSOLI = :P_CDFILSOLI ) AND ( CDFILLAN = :P_CDFILLAN ) AND ( NRSOLICMP = :P_NRSOLICMP ) ) ITEMSOLI WHERE ( PRODUTO.CDPRODUTO = ITEMSOLI.CDPRODUTO AND NRORG = :NRORG ) GROUP BY RTRIM(PRODUTO.NMPRODUTO), PRODUTO.CDARVPROD, PRODUTO.CDPRODUTO, PRODUTO.SGUNIDADE, PRODUTO.NRORG, ITEMSOLI.IDSOLIAPROV, ( CUSTO_PRODUTO(:P_CDFILSOLI, PRODUTO.CDPRODESTO, :P_NRTIPOCUSTO, ITEMSOLI.DTPROGENTR, TRUNC(SYSDATE)) * ( NVL(PRODUTO.VRFATOCONV , 1) ) ) UNION ALL SELECT DISTINCT RTRIM(PRODUTO.NMPRODUTO) NMPRODUTO, PRODUTO.CDARVPROD, PRODUTO.CDPRODUTO, PRODUTO.SGUNIDADE, PRODUTO.NRORG, ITEMSOLI.IDSOLIAPROV, SUM(ITEMSOLI.QTSOLIPROD) QTTOTAL, ( CUSTO_PRODUTO(:P_CDFILSOLI, PRODUTO.CDPRODESTO, :P_NRTIPOCUSTO, ITEMSOLI.DTPROGENTR, TRUNC(SYSDATE)) * ( NVL(PRODUTO.VRFATOCONV , 1) ) ) VRUNIT, SUM((CUSTO_PRODUTO(:P_CDFILSOLI, PRODUTO.CDPRODESTO, :P_NRTIPOCUSTO, ITEMSOLI.DTPROGENTR, TRUNC(SYSDATE)) *(NVL(PRODUTO.VRFATOCONV , 1))) *(NVL(ITEMSOLI.QTSOLIPROD, 0))) AS VRCUSTO FROM PRODUTO, ( SELECT CDPRODUTO, DTPROGENTR, IDSOLIAPROV, TRUNC(QTSOLIPROD, 3) AS QTSOLIPROD FROM ITEMSOLI WHERE ( CDFILSOLI = :P_CDFILSOLI ) AND ( CDFILLAN = :P_CDFILLAN ) AND ( NRSOLICMP = :P_NRSOLICMP ) ) ITEMSOLI WHERE ( PRODUTO.CDPRODUTO = ITEMSOLI.CDPRODUTO AND NRORG = :NRORG ) AND ( :P_ONE_RECORD = 'S' ) GROUP BY RTRIM(PRODUTO.NMPRODUTO), PRODUTO.CDARVPROD, PRODUTO.CDPRODUTO, PRODUTO.SGUNIDADE, PRODUTO.NRORG, ITEMSOLI.IDSOLIAPROV, ( CUSTO_PRODUTO(:P_CDFILSOLI, PRODUTO.CDPRODESTO, :P_NRTIPOCUSTO, ITEMSOLI.DTPROGENTR, TRUNC(SYSDATE)) * ( NVL(PRODUTO.VRFATOCONV , 1) ) ) ORDER BY NMPRODUTO";
   
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
           
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    public function selectCalculaMedia ($row){
        $param = array();
        $param['P_CDFILIAL']   = $row['CDFILSOLI'];
        $param['P_CDPRODUTO']  =   $row['CDPRODUTO'];
       
    $query = "SELECT CAST(QTMEDSOLPROD AS DOUBLE PRECISION) QTMEDSOLPROD FROM PRODPREFMED WHERE (CDFILIAL = :P_CDFILIAL) AND (CDPRODUTO = :P_CDPRODUTO) AND (QTMEDSOLPROD IS NOT NULL OR QTMEDSOLPROD <> 0)";
   
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
           
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getQuantidadeProdutosAprovadosNaSolicitacao($row){
        $query = "SELECT COUNT(1) CONTAPROVACAO
                    FROM ITEMSOLI
                    WHERE CDFILSOLI = :P_CDFILSOLI
                      AND CDFILLAN = :P_CDFILLAN
                      AND NRSOLICMP = :P_NRSOLICMP
                    AND NVL(IDSOLIAPROV, 'N') = 'S'";
        
        $param = array(
            'P_CDFILSOLI' => $row['CDFILSOLI'],
            'P_CDFILLAN'  => $row['CDFILLANCA'],
            'P_NRSOLICMP' => $row['NRSOLICMP'],
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
           
            return $std->fetch();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getUtDestSol($cdfilial)
    {
        $query = "SELECT PARASUPRI.IDULTDESTSOL 
                    FROM PARASUPRI INNER JOIN FILIAL ON PARASUPRI.CDFILIAL = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)
                                                    AND PARASUPRI.NRORG = FILIAL.NRORG
                   WHERE PARASUPRI.IDULTDESTSOL = 'S'
                     AND INSTR(:P_CDFILIAL, FILIAL.CDFILIAL) > 0
                     AND FILIAL.NRORG = :NRORG
                   GROUP BY PARASUPRI.IDULTDESTSOL";
        
        $param = array(
            'P_CDFILIAL' => $cdfilial,
            'NRORG'      => $this->environment->getNrOrg()
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
           
            return $std->fetch();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getItensSolicitacao($row)
    {
        $query = "SELECT ITEMSOLI.DTPROGENTR, ITEMSOLI.IDSOLIAPROV, ITEMSOLI.QTSOLIPROD,
                         ITEMSOLI.IDSOLIAPROV, ITEMSOLI.DTHRAPROVSOL, ITEMSOLI.CDOPERAPROV,
                         ITEMSOLI.CDFILSOLI, ITEMSOLI.CDFILLAN, ITEMSOLI.NRSOLICMP, ITEMSOLI.NRORG
                    FROM ITEMSOLI
                   WHERE ITEMSOLI.CDFILSOLI = :P_CDFILSOLI 
                     AND ITEMSOLI.CDFILLAN  = :P_CDFILLAN 
                     AND ITEMSOLI.NRSOLICMP = :P_NRSOLICMP
                     AND ITEMSOLI.NRORG = :NRORG
                     AND ITEMSOLI.IDSOLIAPROV <> 'S'";
        
        $param = array(
            'P_CDFILSOLI'   => $row['CDFILSOLI'],
            'P_CDFILLAN'    => $row['CDFILLANCA'],
            'P_NRSOLICMP'   => $row['NRSOLICMP'],
            'NRORG'         => $this->environment->getNrOrg()
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
           
            return $std->fetch();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function setAprovaItens($row)
    {
        $query = "UPDATE ITEMSOLI 
                     SET IDSOLIAPROV = 'S',
                         DTHRAPROVSOL = SYSDATE,
                         CDOPERAPROV = :CDOPERAPROV
                   WHERE CDFILSOLI = :P_CDFILSOLI 
                     AND CDFILLAN  = :P_CDFILLAN 
                     AND NRSOLICMP = :P_NRSOLICMP
                     AND NRORG = :NRORG
                     AND IDSOLIAPROV <> 'S'";
        
        $param = array(
            'P_CDFILSOLI'   => $row['CDFILSOLI'],
            'P_CDFILLAN'    => $row['CDFILLANCA'],
            'P_NRSOLICMP'   => $row['NRSOLICMP'],
            'CDOPERAPROV'   => $this->environment->getCdOperador(),
            'NRORG'         => $this->environment->getNrOrg()
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function setReprovaItens($row)
    {
        $query = "UPDATE ITEMSOLI 
                     SET IDSOLIAPROV = 'N',
                         DTHRAPROVSOL = NULL,
                         CDOPERAPROV = NULL
                   WHERE CDFILSOLI = :P_CDFILSOLI 
                     AND CDFILLAN  = :P_CDFILLAN 
                     AND NRSOLICMP = :P_NRSOLICMP
                     AND NRORG = :NRORG
                     AND IDSOLIAPROV = 'S'";
        
        $param = array(
            'P_CDFILSOLI'   => $row['CDFILSOLI'],
            'P_CDFILLAN'    => $row['CDFILLANCA'],
            'P_NRSOLICMP'   => $row['NRSOLICMP'],
            'NRORG'         => $this->environment->getNrOrg()
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getPermissaoAprovReprovacao() {
        $query = "SELECT O.CDOPERADOR, P.NRPERFILACESSO
                    FROM PERFACESSPARC P JOIN OPERADOR O ON (P.NRPARCNEGOCIO = O.NRPARCNEGOCIO AND P.NRORG = O.NRORG)
                                         JOIN ZHU_ACCESS_CONTROL_RULE RU ON P.NRPERFILACESSO = RU.GROUP_ID AND P.NRORG = RU.NRORG
                                         JOIN ZHU_ACCESS_CONTROL_ELEMENT EL ON RU.ACCESS_CONTROL_ELEMENT_ID = EL.ID AND RU.NRORG = EL.NRORG
                   WHERE O.CDOPERADOR = :P_CDOPERADOR
                     AND O.NRORG = :NRORG
                     AND EL.UNIQUE_ID = '5031799863459864241778'
                     AND RU.IS_ACTIVE = 'Y'
                     AND RU.RULE = 'A'";
        
        $param = array(
            'P_CDOPERADOR' => $this->environment->getCdOperador(),
            'NRORG'        => $this->environment->getNrOrg()
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function validaDataProcessamento($row) {
        $param = array(
            "P_CDFILIAL" => $row["CDFILSOLI"]
        );
        
        $query = "SELECT TRUNC(DTPROCESSA) DTPROCESSA
                    FROM
                        (SELECT (
                            CASE PARAMGERAL.IDUTILDTPROC
                            WHEN 'N'
                            THEN SYSDATE
                            ELSE NVL(PARAMFILIAL.DTPROCESSA, SYSDATE)
                            END) DTPROCESSA
                        FROM PARAMGERAL,
                             PARAMFILIAL
                        WHERE PARAMFILIAL.CDFILIAL = :P_CDFILIAL)";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function cmp03100TipoCusto() {
        $query = "SELECT PARAMPLANEJA.CDTPCUSTCMP, NMTIPOCUSTO
                    FROM PARAMPLANEJA, TIPOCUSTO 
                    WHERE PARAMPLANEJA.CDTPCUSTCMP IS NOT NULL
                    AND PARAMPLANEJA.CDTPCUSTCMP = TIPOCUSTO.CDTIPOCUSTO
                    AND PARAMPLANEJA.CDFILIAL = :P_CDFILIAL";
        $param = array(
            "P_CDFILIAL" => $this->environment->getCdFilial()
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function cmp03100ParamCmpSup($P_CDFILIAL) {
        $query = "SELECT IDUTPRODPREF FROM PARCMPSUP WHERE CDFILIAL = :P_CDFILIAL";
        $param = array(
            "P_CDFILIAL" => $P_CDFILIAL
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
}